

#include "stdafx.h"
#include "AVIMuxFilter.h"

#include <aviriff.h>


typedef struct _avioldindex_entry
{
      DWORD   dwChunkId;
      DWORD   dwFlags;
/*
      #ifndef AVIIF_LIST
      #define AVIIF_LIST       0x00000001
      #define AVIIF_KEYFRAME   0x00000010
      #endif
     
      #define AVIIF_NO_TIME    0x00000100
      #define AVIIF_COMPRESSOR 0x0FFF0000  // unused?
*/
      DWORD   dwOffset;    // offset of riff chunk header for the data
      DWORD   dwSize;      // size of the data (excluding riff header size)
} AVIOLDINDEX_ENTRY;



#define TEMPLATE_NAME	(L"PeCaTV AVI Mux")
#define FILTER_NAME		(TEMPLATE_NAME)
#define VIDEO_INPUT_PIN_NAME	(L"Video In")
#define AUDIO_INPUT_PIN_NAME	(L"Audio In")

// s̃fBA^Cv
const AMOVIESETUP_MEDIATYPE sudPinVideoTypes =
{
	&MEDIATYPE_Video, &MEDIASUBTYPE_NULL
};

const AMOVIESETUP_MEDIATYPE sudPinAudioTypes =
{
	&MEDIATYPE_Audio, &MEDIASUBTYPE_NULL
};

// sݒ
const AMOVIESETUP_PIN sudPins[2] =
{
	{
		VIDEO_INPUT_PIN_NAME, // s̖O (p~)
		FALSE, // TRUE ̏ꍇAtB^͂̃s̓͂_O
		FALSE, // TRUE ̏ꍇÃs͏o̓sƂȂ
		FALSE, // TRUE ̏ꍇAtB^̃s̃CX^X 1 ȂƂ
		FALSE, // TRUE ̏ꍇAtB^̎ނ̃s̃CX^X𕡐쐬ł
		&CLSID_NULL, // ̃sڑtB^̃NXʎq (CLSID) (p~)
		NULL, // ̃sڑs̖O (p~)
		1, // ̃sT|[g郁fBA^Cv
		&sudPinVideoTypes // \̂̔zւ̃|C^Bz̃TCY nMediaTypes
	},
	{
		AUDIO_INPUT_PIN_NAME,
		FALSE,
		FALSE, // ̓s
		FALSE,
		FALSE,
		&CLSID_NULL,
		NULL,
		1,
		&sudPinAudioTypes
	},
};

// tB^ݒ
const AMOVIESETUP_FILTER afFilterInfo =
{
	&__uuidof(CAVIMuxFilter), // tB^̃NXʎq
	FILTER_NAME, // tB^̖O
	MERIT_DO_NOT_USE, // tB^̃bg
	_countof(sudPins), // tB^̃s
	sudPins // sݒ
};



//===============================================================================================
// VideoInputPin
//===============================================================================================
VideoInputPin::VideoInputPin(LPCTSTR pName, CBaseFilter *pFilter , CCritSec *pLock, HRESULT *phr)
	: CBaseInputPin(pName, pFilter, pLock, phr, pName)
{
}
VideoInputPin::~VideoInputPin()
{
}

STDMETHODIMP VideoInputPin::EndOfStream(void)
{
	CAutoLock lock(m_pLock);

	m_EOS = true;

	return CheckStreaming();
}

STDMETHODIMP VideoInputPin::Receive(IMediaSample *pSample)
{
	CAutoLock lock(m_pLock);

	HRESULT hr;

	hr = __super::Receive(pSample);

	if (hr != S_OK)
	{
		return hr;
	}

	if (m_EOS)
	{
		return S_FALSE;
	}

	return ((CAVIMuxFilter*)m_pFilter)->ReceiveVideo(pSample);
}

HRESULT VideoInputPin::Active(void)
{
	m_EOS = false;
	return __super::Active();
}

HRESULT VideoInputPin::GetMediaType(int iPosition, CMediaType *pMediaType)
{
	if (iPosition < 0)
	{
		return E_INVALIDARG;
	}
	return VFW_S_NO_MORE_ITEMS;
}

HRESULT VideoInputPin::CheckMediaType(const CMediaType *pmt)
{
	if (pmt->majortype == MEDIATYPE_Video &&
		pmt->formattype == FORMAT_VideoInfo &&
		pmt->cbFormat >= sizeof(VIDEOINFOHEADER) &&
		pmt->pbFormat != NULL)
	{
		m_vih = *((VIDEOINFOHEADER *)pmt->pbFormat);
		return S_OK;
	}
	return S_FALSE;
}


//===============================================================================================
// AudioInputPin
//===============================================================================================
AudioInputPin::AudioInputPin(LPCTSTR pName, CBaseFilter *pFilter , CCritSec *pLock, HRESULT *phr)
	: CBaseInputPin(pName, pFilter, pLock, phr, pName)
{
}
AudioInputPin::~AudioInputPin()
{
}

STDMETHODIMP AudioInputPin::EndOfStream(void)
{
	CAutoLock lock(m_pLock);

	m_EOS = true;

	return CheckStreaming();
}

STDMETHODIMP AudioInputPin::Receive(IMediaSample *pSample)
{
	CAutoLock lock(m_pLock);

	HRESULT hr;

	hr = __super::Receive(pSample);

	if (hr != S_OK)
	{
		return hr;
	}

	if (m_EOS)
	{
		return S_FALSE;
	}

	return ((CAVIMuxFilter*)m_pFilter)->ReceiveAudio(pSample);
}


HRESULT AudioInputPin::Active(void)
{
	m_EOS = false;
	return __super::Active();
}

HRESULT AudioInputPin::GetMediaType(int iPosition, CMediaType *pMediaType)
{
	if (iPosition < 0)
	{
		return E_INVALIDARG;
	}
	return VFW_S_NO_MORE_ITEMS;
}

HRESULT AudioInputPin::CheckMediaType(const CMediaType *pmt)
{
	if (pmt->majortype == MEDIATYPE_Audio &&
		pmt->formattype == FORMAT_WaveFormatEx &&
		pmt->cbFormat >= sizeof(WAVEFORMATEX) &&
		pmt->pbFormat != NULL)
	{
		m_wfx = *((WAVEFORMATEX *)pmt->pbFormat);
		return S_OK;
	}
	return S_FALSE;
}


//===============================================================================================
// CAVIMuxFilter
//===============================================================================================
CUnknown *CAVIMuxFilter::CreateInstance(LPUNKNOWN pUnk, HRESULT *phr)
{
	CAVIMuxFilter *pNewFilter = new CAVIMuxFilter(pUnk, phr);
	if (pNewFilter == NULL)
	{
		*phr = E_OUTOFMEMORY;
	}
	else
	{
		*phr = S_OK;
	}
	return pNewFilter;
}

CAVIMuxFilter::CAVIMuxFilter(LPUNKNOWN pUnk, HRESULT *phr)
	: m_FilterLock()
	, m_pVideoInput(NULL)
	, m_pAudioInput(NULL)
	, CBaseFilter(FILTER_NAME, pUnk, &m_FilterLock, __uuidof(CAVIMuxFilter))
{
	*phr = S_OK;

	// rfI̓s
	m_pVideoInput = new VideoInputPin(VIDEO_INPUT_PIN_NAME, this, &m_FilterLock, phr);
	if (FAILED(*phr) || m_pVideoInput == NULL)
	{
		return;
	}

	// I[fBI̓s
	m_pAudioInput = new AudioInputPin(AUDIO_INPUT_PIN_NAME, this, &m_FilterLock, phr);
	if (FAILED(*phr) || m_pAudioInput == NULL)
	{
		delete m_pVideoInput;
		return;
	}
}

CAVIMuxFilter::~CAVIMuxFilter()
{
	// s̊J
	delete m_pVideoInput;
	delete m_pAudioInput;
}


FILE *m_pFile = NULL;
DWORD m_dwRiffSize;
DWORD m_dwFrames;
DWORD m_dwSamples;
DWORD m_dwSampleSize;

std::vector<AVIOLDINDEX_ENTRY> m_vecIndexEntry;

HRESULT CAVIMuxFilter::StartStreaming()
{
	if (::_wfopen_s(&m_pFile, L"E:\\test.avi", L"wb+"))
	{
		return TRUE;
	}



	::rewind(m_pFile);
	::fseek(m_pFile, 2048, SEEK_SET); // Header2KB΂ăf[^ވʒuݒ
	m_dwRiffSize = 2048;
	m_dwFrames = 0;
	m_dwSamples = 0;
	m_dwSampleSize = 0;
	m_vecIndexEntry.clear();

	return S_OK;
}

HRESULT CAVIMuxFilter::StopStreaming()
{
/*
	DWORD dwHeader;
	dwHeader =	sizeof(RIFFLIST) +			// RIFF AVI
				sizeof(RIFFLIST) +			// LIST hdrl
				sizeof(AVIMAINHEADER)		// avih (AVIMAINHEADER)

				sizeof(RIFFLIST) +			// LIST strl
				sizeof(AVISTREAMHEADER) +	// strh (AVISTREAMHEADER)
				sizeof(RIFFCHUNK) +			// strf
				m_pVideoInput->CurrentMediaType().FormatLength() +

				sizeof(RIFFLIST) +			// LIST strl
				sizeof(AVISTREAMHEADER) +	// strh (AVISTREAMHEADER)
				sizeof(RIFFCHUNK) +			// strf
				m_pAudioInput->CurrentMediaType().FormatLength() +
*/

	DWORD dwHeader = 0;

	RIFFLIST list;
	RIFFCHUNK chunk;
	AVIMAINHEADER avih;
	AVISTREAMHEADER strh;

	::rewind(m_pFile);

	// RIFF AVI
	list.fcc = FCC('RIFF');
	list.cb = m_dwRiffSize + sizeof(RIFFCHUNK) + (sizeof(AVIOLDINDEX_ENTRY) * (DWORD)m_vecIndexEntry.size()) - 8;
	list.fccListType = FCC('AVI ');

	::fwrite(&list, sizeof(RIFFLIST), 1, m_pFile);
	dwHeader += sizeof(RIFFLIST);

	// LIST hdrl
	list.fcc = FCC('LIST');
	list.cb = 4 + sizeof(AVIMAINHEADER)
		+ sizeof(RIFFLIST) + sizeof(AVISTREAMHEADER) + sizeof(RIFFCHUNK) + sizeof(BITMAPINFOHEADER)
		+ sizeof(RIFFLIST) + sizeof(AVISTREAMHEADER) + sizeof(RIFFCHUNK) + sizeof(WAVEFORMATEX);
	list.fccListType = FCC('hdrl');

	::fwrite(&list, sizeof(RIFFLIST), 1, m_pFile);
	dwHeader += sizeof(RIFFLIST);

	// avih (AVIMAINHEADER)
	avih.fcc = FCC('avih');
	avih.cb = sizeof(AVIMAINHEADER) - 8;                     // size of this structure -8
	avih.dwMicroSecPerFrame = (DWORD)(m_pVideoInput->GetVideoInfoHeader().AvgTimePerFrame / 100);     // frame display rate (or 0L)
	avih.dwMaxBytesPerSec = m_pVideoInput->GetVideoInfoHeader().dwBitRate;       // max. transfer rate
	avih.dwPaddingGranularity = 0;   // pad to multiples of this size; normally 2K.
	avih.dwFlags = AVIF_HASINDEX | AVIF_WASCAPTUREFILE;
	avih.dwTotalFrames = m_dwFrames;          // # frames in first movi list
	avih.dwInitialFrames = 0;
	avih.dwStreams = 2;
	avih.dwSuggestedBufferSize = 0;
	avih.dwWidth = m_pVideoInput->GetVideoInfoHeader().bmiHeader.biWidth;
	avih.dwHeight = m_pVideoInput->GetVideoInfoHeader().bmiHeader.biHeight;
	avih.dwReserved[0] = 0;
	avih.dwReserved[1] = 0;
	avih.dwReserved[2] = 0;
	avih.dwReserved[3] = 0;

	::fwrite(&avih, sizeof(AVIMAINHEADER), 1, m_pFile);
	dwHeader += sizeof(AVIMAINHEADER);


	//--------------------------------------------------------------------------------------
	// LIST strl
	list.fcc = FCC('LIST');
	list.cb = 4 + sizeof(AVISTREAMHEADER) + sizeof(RIFFCHUNK) + sizeof(BITMAPINFOHEADER);
	list.fccListType = FCC('strl');

	::fwrite(&list, sizeof(RIFFLIST), 1, m_pFile);
	dwHeader += sizeof(RIFFLIST);

	// strh (AVISTREAMHEADER)
	strh.fcc = FCC('strh');
	strh.cb = sizeof(AVISTREAMHEADER) - 8;           // size of this structure - 8

	strh.fccType = streamtypeVIDEO;      // stream type codes
	strh.fccHandler = m_pVideoInput->GetVideoInfoHeader().bmiHeader.biCompression;
	strh.dwFlags = 0;
	strh.wPriority = 0;
	strh.wLanguage = 0;
	strh.dwInitialFrames = 0;
	strh.dwScale = (DWORD)m_pVideoInput->GetVideoInfoHeader().AvgTimePerFrame;
	strh.dwRate = 10000000;       // dwRate/dwScale is stream tick rate in ticks/sec
	strh.dwStart = 0;
	strh.dwLength = m_dwFrames;
	strh.dwSuggestedBufferSize = 0;
	strh.dwQuality = 0;
	strh.dwSampleSize = 0;
	strh.rcFrame.left = 0;
	strh.rcFrame.top = 0;
	strh.rcFrame.right = (short)m_pVideoInput->GetVideoInfoHeader().bmiHeader.biWidth;
	strh.rcFrame.bottom = (short)m_pVideoInput->GetVideoInfoHeader().bmiHeader.biHeight;

	::fwrite(&strh, sizeof(AVISTREAMHEADER), 1, m_pFile);
	dwHeader += sizeof(AVISTREAMHEADER);

	// strf
	chunk.fcc = FCC('strf');
	chunk.cb = sizeof(BITMAPINFOHEADER);

	::fwrite(&chunk, sizeof(RIFFCHUNK), 1, m_pFile);
	dwHeader += sizeof(RIFFCHUNK);

	::fwrite(&(m_pVideoInput->GetVideoInfoHeader().bmiHeader), chunk.cb, 1, m_pFile);
	dwHeader += sizeof(BITMAPINFOHEADER);


	//--------------------------------------------------------------------------------------
	// LIST strl
	list.fcc = FCC('LIST');
	list.cb = 4 + sizeof(AVISTREAMHEADER) + sizeof(RIFFCHUNK) + sizeof(WAVEFORMATEX);
	list.fccListType = FCC('strl');

	::fwrite(&list, sizeof(RIFFLIST), 1, m_pFile);
	dwHeader += sizeof(RIFFLIST);

	// strh (AVISTREAMHEADER)
	strh.fcc = FCC('strh');
	strh.cb = sizeof(AVISTREAMHEADER) - 8;           // size of this structure - 8

	strh.fccType = streamtypeAUDIO;      // stream type codes
	strh.fccHandler = 1;
	strh.dwFlags = 0;
	strh.wPriority = 0;
	strh.wLanguage = 0;
	strh.dwInitialFrames = 0;
	strh.dwScale = m_pAudioInput->GetWaveFormatEX().nBlockAlign;
	strh.dwRate = m_pAudioInput->GetWaveFormatEX().nAvgBytesPerSec;       // dwRate/dwScale is stream tick rate in ticks/sec
	strh.dwStart = 0;
	strh.dwLength = (DWORD)((double)(m_dwFrames * m_pAudioInput->GetWaveFormatEX().nSamplesPerSec) / ((double)10000000.0 / m_pVideoInput->GetVideoInfoHeader().AvgTimePerFrame));
	strh.dwSuggestedBufferSize = m_dwSampleSize;
	strh.dwQuality = 0;
	strh.dwSampleSize = m_pAudioInput->GetWaveFormatEX().nBlockAlign;
	strh.rcFrame.left = 0;
	strh.rcFrame.top = 0;
	strh.rcFrame.right = 0;
	strh.rcFrame.bottom = 0;

	::fwrite(&strh, sizeof(AVISTREAMHEADER), 1, m_pFile);
	dwHeader += sizeof(AVISTREAMHEADER);

	// strf
	chunk.fcc = FCC('strf');
	chunk.cb = sizeof(WAVEFORMATEX);

	::fwrite(&chunk, sizeof(RIFFCHUNK), 1, m_pFile);
	dwHeader += sizeof(RIFFCHUNK);

	::fwrite(&(m_pAudioInput->GetWaveFormatEX()), chunk.cb, 1, m_pFile);
	dwHeader += sizeof(WAVEFORMATEX);
	//--------------------------------------------------------------------------------------

	// JUNK
	chunk.fcc = FCC('JUNK');
	chunk.cb = 2048 - (dwHeader + sizeof(RIFFCHUNK) + sizeof(RIFFLIST));

	::fwrite(&chunk, sizeof(RIFFCHUNK), 1, m_pFile);

	LPBYTE pJunk = (LPBYTE)_malloca(chunk.cb);
	::memset(pJunk, 0, chunk.cb);
	::fwrite(pJunk, chunk.cb, 1, m_pFile);
	::_freea(pJunk);

	// LIST hdrl
	list.fcc = FCC('LIST');
	list.cb = m_dwRiffSize - 2048 + 4;
	list.fccListType = FCC('movi');
	::fwrite(&list, sizeof(RIFFLIST), 1, m_pFile);

	//---------------------------------------------------------------------------------------

	::fseek(m_pFile, m_dwRiffSize, SEEK_SET);

	chunk.fcc = FCC('idx1');
	chunk.cb = (sizeof(AVIOLDINDEX_ENTRY) * (DWORD)m_vecIndexEntry.size());
	::fwrite(&chunk, sizeof(RIFFCHUNK), 1, m_pFile);

	std::vector<AVIOLDINDEX_ENTRY>::iterator it;
	for (it = m_vecIndexEntry.begin(); it != m_vecIndexEntry.end(); ++it)
	{
		::fwrite(&(*it), sizeof(AVIOLDINDEX_ENTRY), 1, m_pFile);
	}

	::fclose(m_pFile);

	return S_OK;
}

static LONGLONG test = 0;
HRESULT CAVIMuxFilter::ReceiveVideo(IMediaSample *pSample)
{
	LONG lDataLength = pSample->GetActualDataLength();
	LPBYTE pData = NULL;
	RIFFCHUNK chunk;


	AVIOLDINDEX_ENTRY entry;
	entry.dwChunkId = FCC('00dc');
	entry.dwFlags = AVIIF_KEYFRAME;
	entry.dwOffset = (m_dwRiffSize - 2048) + 4;
	entry.dwSize = lDataLength;

	m_vecIndexEntry.push_back(entry);

	if (true/*test%4 == 0*/)
	{
		pSample->GetPointer(&pData);

		chunk.fcc = FCC('00dc');
		chunk.cb = lDataLength;

		::fwrite(&chunk, sizeof(RIFFCHUNK), 1, m_pFile);
		::fwrite(pData, lDataLength, 1, m_pFile);
		m_dwRiffSize += lDataLength + sizeof(RIFFCHUNK);
	}
	else
	{
		chunk.fcc = FCC('00dc');
		chunk.cb = 0;
		::fwrite(&chunk, sizeof(RIFFCHUNK), 1, m_pFile);
		m_dwRiffSize += sizeof(RIFFCHUNK);
	}

	test++;

	m_dwFrames++;


	return S_OK;
}

HRESULT CAVIMuxFilter::ReceiveAudio(IMediaSample *pSample)
{

	LONG lDataLength = pSample->GetActualDataLength();
	LPBYTE pData = NULL;
	RIFFCHUNK chunk;

	AVIOLDINDEX_ENTRY entry;
	entry.dwChunkId = FCC('01wb');
	entry.dwFlags = AVIIF_KEYFRAME;
	entry.dwOffset = (m_dwRiffSize - 2048) + 4;
	entry.dwSize = lDataLength;

	m_vecIndexEntry.push_back(entry);


	pSample->GetPointer(&pData);

	chunk.fcc = FCC('01wb');
	chunk.cb = lDataLength;

	::fwrite(&chunk, sizeof(RIFFCHUNK), 1, m_pFile);
	::fwrite(pData, lDataLength, 1, m_pFile);
	m_dwRiffSize += lDataLength + sizeof(RIFFCHUNK);

	if (m_dwSampleSize < (unsigned)lDataLength)
	{
		m_dwSampleSize = lDataLength;
	}

	m_dwSamples++;

	return S_OK;
}


int CAVIMuxFilter::GetPinCount(void)
{
	return 2;
}

CBasePin *CAVIMuxFilter::GetPin(int n)
{
	switch(n)
	{
	case 0:
		return (CBasePin*)m_pVideoInput;
	case 1:
		return (CBasePin*)m_pAudioInput;
	}
	return NULL;
}

STDMETHODIMP CAVIMuxFilter::Stop()
{
    CAutoLock lock(&m_FilterLock);
	HRESULT hr;

    if (m_State == State_Stopped)
	{
        return NOERROR;
    }

	if (m_pVideoInput)
	{
		m_pVideoInput->Inactive();
	}

	if (m_pAudioInput)
	{
		m_pAudioInput->Inactive();
	}

    hr = StopStreaming();
    if (SUCCEEDED(hr))
	{
		m_State = State_Stopped;
    }
	return hr;
}

STDMETHODIMP CAVIMuxFilter::Pause()
{
	CAutoLock lock(&m_FilterLock);
	HRESULT hr = NOERROR;

	if (m_State == State_Stopped)
	{
		hr = StartStreaming();
	}

	if (SUCCEEDED(hr))
	{
		hr = __super::Pause();
	}
	return hr;
}

