#pragma once

/*
// {7574EC5F-2B11-4e38-AC85-38F8C0D29D09}
DEFINE_GUID(CLSID_PeCaTVAVIMux, 
0x7574ec5f, 0x2b11, 0x4e38, 0xac, 0x85, 0x38, 0xf8, 0xc0, 0xd2, 0x9d, 0x9);
*/

class __declspec(uuid("{7574EC5F-2B11-4e38-AC85-38F8C0D29D09}")) CAVIMuxFilter;

class VideoInputPin : public CBaseInputPin
{
public:
	VideoInputPin(LPCTSTR pName, CBaseFilter *pFilter, CCritSec *pLock, HRESULT *phr);
	~VideoInputPin();

	CMediaType& CurrentMediaType() { return m_mt; };
	VIDEOINFOHEADER& GetVideoInfoHeader() { return m_vih; };

	//IPin
	STDMETHODIMP EndOfStream(void);
	STDMETHODIMP Receive(IMediaSample *pSample);
	bool GetEOS() { return m_EOS; }

	//CBaseInputPin
	HRESULT Active(void);
	HRESULT GetMediaType(int iPosition, CMediaType *pMediaType);
	HRESULT CheckMediaType(const CMediaType *pmt);

private:
	bool m_EOS;
	VIDEOINFOHEADER m_vih;
};

class AudioInputPin : public CBaseInputPin
{
public:
	AudioInputPin(LPCTSTR pName, CBaseFilter *pFilter, CCritSec *pLock, HRESULT *phr);
	~AudioInputPin();

	CMediaType& CurrentMediaType() { return m_mt; };
	WAVEFORMATEX& GetWaveFormatEX() { return m_wfx; };

	//IPin
	STDMETHODIMP EndOfStream(void);
	STDMETHODIMP Receive(IMediaSample *pSample);
	bool GetEOS() { return m_EOS; }

	//CBaseInputPin
	HRESULT Active(void);
	HRESULT GetMediaType(int iPosition, CMediaType *pMediaType);
	HRESULT CheckMediaType(const CMediaType *pmt);

private:
	bool m_EOS;
	WAVEFORMATEX m_wfx;
};

class CAVIMuxFilter : public CBaseFilter
{
	CCritSec m_FilterLock; // tB^bN
	VideoInputPin *m_pVideoInput; // ̓s
	AudioInputPin *m_pAudioInput; // ̓s

public:
	static CUnknown * WINAPI CreateInstance(LPUNKNOWN pUnk, HRESULT *phr);
	CAVIMuxFilter(LPUNKNOWN pUnk, HRESULT *phr);
	~CAVIMuxFilter();

	VideoInputPin *GetVideoInputPin() const { return m_pVideoInput; }
	AudioInputPin *GetAudioInputPin() const { return m_pAudioInput; }

	HRESULT StartStreaming();
	HRESULT StopStreaming();

	HRESULT ReceiveVideo(IMediaSample *pSample);
	HRESULT ReceiveAudio(IMediaSample *pSample);


	// CBaseFilter
	int GetPinCount(void);
	CBasePin *GetPin(int n);

	STDMETHODIMP Stop();
	STDMETHODIMP Pause();
};
