#include "StdAfx.h"
#include "CaptureFilterInfo.h"

#define FAILED_TRACE TRACE("<FAILED> %s (%d) %s \n", __FILE__, __LINE__, __FUNCTION__)

CCaptureFilterInfo::CCaptureFilterInfo(void)
{
	m_bCoInitialize = FALSE;
}

CCaptureFilterInfo::~CCaptureFilterInfo(void)
{
}

BOOL CCaptureFilterInfo::Init(void)
{
	// COM
	UnInit();

	HRESULT hr = S_OK;
	
	hr = CoInitialize(NULL);

	if (FAILED(hr))
	{
		return FALSE;
	}

	m_bCoInitialize = TRUE;
	return TRUE;
}

void CCaptureFilterInfo::UnInit(void)
{
	// COMI
	if (m_bCoInitialize)
	{
		m_bCoInitialize = FALSE;
		CoUninitialize();
	}
}

BOOL CCaptureFilterInfo::IsInit(void)
{
	return m_bCoInitialize;
}

BOOL CCaptureFilterInfo::MakeVideoFourCC(const GUID &subtype, const DWORD biCompression, std::wstring *pwsOutput)
{
	if (pwsOutput == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}

	const GUID guid[] = {
		MEDIASUBTYPE_AYUV,
		MEDIASUBTYPE_UYVY,
		MEDIASUBTYPE_Y411,
		MEDIASUBTYPE_Y41P,
		MEDIASUBTYPE_Y211,
		MEDIASUBTYPE_YUY2,
		MEDIASUBTYPE_YVYU,
		MEDIASUBTYPE_YUYV,

		MEDIASUBTYPE_IF09,
		MEDIASUBTYPE_IYUV,
		MEDIASUBTYPE_YV12,
		MEDIASUBTYPE_YVU9,

		MEDIASUBTYPE_RGB1,
		MEDIASUBTYPE_RGB4,
		MEDIASUBTYPE_RGB8,
		MEDIASUBTYPE_RGB555,
		MEDIASUBTYPE_RGB565,
		MEDIASUBTYPE_RGB24,
		MEDIASUBTYPE_RGB32,
		MEDIASUBTYPE_ARGB32,
	};

	LPCWSTR fourcc[] = {
		L"AYUV",
		L"UYVY",
		L"Y411",
		L"Y41P",
		L"Y211",
		L"YUY2",
		L"YVYU",
		L"YUYV",

		L"IF09",
		L"IYUV",
		L"YV12",
		L"YVU9",

		L"RGB1",
		L"RGB4",
		L"RGB8",
		L"RGB555",
		L"RGB565",
		L"RGB24",
		L"RGB32",
		L"ARGB32",
	};

	int count = (int)(sizeof(guid) / sizeof(guid[0]));

	for (int i = 0; i < count; i++)
	{
		if (IsEqualGUID(subtype, guid[i]) == TRUE)
		{
			*pwsOutput = fourcc[i];
			return TRUE;
		}
	}

	switch (biCompression)
	{
	case BI_RGB:
		*pwsOutput = L"BI_RGB";
		break;

	case BI_RLE8:
		*pwsOutput = L"BI_RLE8";
		break;

	case BI_RLE4:
		*pwsOutput = L"BI_RLE4";
		break;

	case BI_BITFIELDS:
		*pwsOutput = L"BI_BITFIELDS";
		break;

	case BI_JPEG:
		*pwsOutput = L"BI_JPEG";
		break;

	case BI_PNG:
		*pwsOutput = L"BI_PNG";
		break;
	default:
		{
			std::wstring wsTmp;
			char buf[8];
			buf[0] = (char)(BYTE)( biCompression & 0x000000FF);
			buf[1] = (char)(BYTE)((biCompression & 0x0000FF00) >> 8);
			buf[2] = (char)(BYTE)((biCompression & 0x00FF0000) >> 16);
			buf[3] = (char)(BYTE)((biCompression & 0xFF000000) >> 24);
			buf[4] = '\0';

			const int nBuff = 32;
			WCHAR wszBuff[nBuff];

			::MultiByteToWideChar(CP_ACP, 0, buf, -1, wszBuff, nBuff);


			wsTmp = wszBuff;

			std::transform(wsTmp.begin(), wsTmp.end(), wsTmp.begin(), ToUpper());

			*pwsOutput = wsTmp;

			
		}
	}

	return TRUE;
}

BOOL CCaptureFilterInfo::MakeAudioFourCC(const GUID &subtype, std::wstring *pwsOutput)
{
	if (pwsOutput == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}

	const GUID guid[] = {
		MEDIASUBTYPE_PCM,
		MEDIASUBTYPE_MPEG1Packet,
		MEDIASUBTYPE_MPEG1Payload,
		MEDIASUBTYPE_MPEG2_AUDIO,
		MEDIASUBTYPE_DVD_LPCM_AUDIO,
		MEDIASUBTYPE_MPEG2_AUDIO,
		MEDIASUBTYPE_DRM_Audio,
		MEDIASUBTYPE_IEEE_FLOAT,
		MEDIASUBTYPE_DOLBY_AC3,
		MEDIASUBTYPE_DOLBY_AC3_SPDIF,
		MEDIASUBTYPE_RAW_SPORT,
		MEDIASUBTYPE_SPDIF_TAG_241h,
	};

	LPCWSTR fourcc[] = {
		L"PCM",
		L"MPEG1Packet",
		L"MPEG1Payload",
		L"MPEG2_AUDIO",
		L"DVD_LPCM_AUDIO",
		L"MPEG2_AUDIO",
		L"DRM_Audio",
		L"IEEE_FLOAT",
		L"DOLBY_AC3",
		L"DOLBY_AC3_SPDIF",
		L"RAW_SPORT",
		L"SPDIF_TAG_241h",
	};

	int count = (int)(sizeof(guid) / sizeof(guid[0]));

	for (int i = 0; i < count; i++)
	{
		if (IsEqualGUID(subtype, guid[i]) == TRUE)
		{
			*pwsOutput = fourcc[i];
			return TRUE;
		}
	}

	*pwsOutput = L"Unknown";

	return TRUE;
}


BOOL CCaptureFilterInfo::CreateBuilder(IGraphBuilder** ppGB, ICaptureGraphBuilder2** ppCGB2)
{
	if (ppGB == NULL || ppCGB2 == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}

	HRESULT hr;

	// IGraphBuilder擾
	hr = CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC,
		IID_IGraphBuilder, (LPVOID *)ppGB);

	if (FAILED(hr) || *ppGB == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}

	// ICaptureGraphBuilder2擾
	hr = CoCreateInstance(CLSID_CaptureGraphBuilder2, NULL, CLSCTX_INPROC, 
		IID_ICaptureGraphBuilder2, (LPVOID *)ppCGB2);

	if (FAILED(hr) || *ppCGB2 == NULL)
	{
		(*ppGB)->Release();
		*ppGB = NULL;
		FAILED_TRACE;
		return FALSE;
	}

	// OtZbg
	hr = (*ppCGB2)->SetFiltergraph(*ppGB);

	if (FAILED(hr))
	{
		(*ppGB)->Release();
		*ppGB = NULL;
		(*ppCGB2)->Release();
		*ppCGB2 = NULL;
		FAILED_TRACE;
		return FALSE;
	}

	return TRUE;
}

BOOL CCaptureFilterInfo::GetPinMediaType(IPin *pPin, GUID *pMediaType)
{
	if (pPin == NULL || pMediaType == NULL)
	{
		return FALSE;
	}

	HRESULT hr;
	CComPtr<IEnumMediaTypes> pEnum;

	hr = pPin->EnumMediaTypes(&pEnum);
	if (FAILED(hr) || pEnum == NULL)
	{
		return FALSE;
	}

	hr = pEnum->Reset();
	if (FAILED(hr))
	{
		return FALSE;
	}

	AM_MEDIA_TYPE *pMT = NULL;
	ULONG nFetched = 0;
	while (pEnum->Next(1, &pMT, &nFetched) == S_OK)
	{
		if (pMT != NULL)
		{
			*pMediaType = pMT->majortype;
			DeleteMediaType(pMT);
			return TRUE;
		}
		DeleteMediaType(pMT);
	}

	return FALSE;
}

BOOL CCaptureFilterInfo::GetPinMediaType(IPin *pPin, std::wstring &wsMediaType)
{
	FAILED_TRACE;
	if (pPin == NULL)
	{
		return FALSE;
	}

	GUID mediatype;
	if (!GetPinMediaType(pPin, &mediatype))
	{
		return FALSE;
	}

	const GUID guid[] = {
		MEDIATYPE_AnalogAudio,
		MEDIATYPE_AnalogVideo,
		MEDIATYPE_Audio,
		MEDIATYPE_AUXLine21Data,
		MEDIATYPE_File,
		MEDIATYPE_Interleaved,
		MEDIATYPE_LMRT,
		MEDIATYPE_Midi,
		MEDIATYPE_MPEG2_PES,
		MEDIATYPE_ScriptCommand,
		MEDIATYPE_Stream,
		MEDIATYPE_Text,
		MEDIATYPE_Timecode,
		MEDIATYPE_URL_STREAM,
		MEDIATYPE_Video,
		MEDIASUBTYPE_None,
		MEDIATYPE_NULL,
	};

	LPCWSTR name[] = {
		L"AnalogAudio",
		L"AnalogVideo",
		L"Audio",
		L"AUXLine21Data",
		L"File",
		L"Interleaved",
		L"LMRT",
		L"Midi",
		L"MPEG2_PES",
		L"ScriptCommand",
		L"Stream",
		L"Text",
		L"Timecode",
		L"URL_STREAM",
		L"Video",
		L"None",
		L"NULL",
	};


	BOOL found = FALSE;
	int count = sizeof(guid) / sizeof(guid[0]);
	for (int i = 0; i < count; i++)
	{
		if (::IsEqualGUID(mediatype, guid[i]))
		{
			wsMediaType = name[i];
			found = TRUE;
			break;
		}
	}

	if (!found)
	{
		wsMediaType = L"Unknown";
	}


	return TRUE;
}

BOOL CCaptureFilterInfo::GetPinCategory(IPin *pPin, GUID *pPinCategory)
{
	if (pPin == NULL || pPinCategory == NULL)
	{
		return FALSE;
	}

	HRESULT hr;
	CComPtr<IKsPropertySet> pKs;
	hr = pPin->QueryInterface(IID_IKsPropertySet, (void **)&pKs);
	if (FAILED(hr) || pKs == NULL)
	{
		// s IKsPropertySet T|[gĂȂB
		return FALSE;
	}

	// s JeS擾悤ƂB
	// ̌ĂяoɐƁApPinCategory ɂ̓JeS GUID i[B
	DWORD cbReturned;
	hr = pKs->Get(AMPROPSETID_Pin, AMPROPERTY_PIN_CATEGORY, NULL, 0, 
		pPinCategory, sizeof(GUID), &cbReturned);

	if (FAILED(hr))
	{
		return FALSE;
	}

	return TRUE;
}


BOOL CCaptureFilterInfo::GetPinCategory(IPin *pPin, std::wstring &wsCategory)
{
	if (pPin == NULL)
	{
		return FALSE;
	}

	GUID category;

	if (!GetPinCategory(pPin, &category))
	{
		return FALSE;
	}


	const GUID guid[] = {
		PIN_CATEGORY_ANALOGVIDEOIN,
		PIN_CATEGORY_CAPTURE,
		PIN_CATEGORY_CC,
		PIN_CATEGORY_EDS,
		PIN_CATEGORY_NABTS,
		PIN_CATEGORY_PREVIEW,
		PIN_CATEGORY_STILL,
		PIN_CATEGORY_TELETEXT,
		PIN_CATEGORY_TIMECODE,
		PIN_CATEGORY_VBI,
		PIN_CATEGORY_VIDEOPORT,
		PIN_CATEGORY_VIDEOPORT_VBI,
		GUID_NULL,
	};

	LPCWSTR name[] = {
		L"ANALOGVIDEOIN",
		L"CAPTURE",
		L"CC",
		L"EDS",
		L"NABTS",
		L"PREVIEW",
		L"STILL",
		L"TELETEXT",
		L"TIMECODE",
		L"VBI",
		L"VIDEOPORT",
		L"VIDEOPORT_VBI",
		L"NULL",
	};

	BOOL found = FALSE;
	int count = sizeof(guid) / sizeof(guid[0]);
	for (int i = 0; i < count; i++)
	{
		if (::IsEqualGUID(category, guid[i]))
		{
			wsCategory = name[i];
			found = TRUE;
			break;
		}
	}

	if (!found)
	{
		wsCategory = L"UNKNOWN";
	}

	return TRUE;
}

BOOL CCaptureFilterInfo::HasAudioCapturePin(std::vector<CPinInfo> &vecInfo)
{
	std::vector<CPinInfo>::iterator it;
	for (it = vecInfo.begin(); it != vecInfo.end(); ++it)
	{
		if (!it->bInput)
		{
			if (it->wsCategory == L"CAPTURE" && it->wsMediaType == L"Audio")
			{
				return TRUE;
			}
		}
	}
	return FALSE;
}

BOOL CCaptureFilterInfo::GetPinInfo(IBaseFilter *pFilter, std::vector<CPinInfo> &vecInfo)
{
	if (pFilter == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}

	HRESULT hr;
    CComPtr<IEnumPins>	pEnum;
    CComPtr<IPin>		pPin;
	ULONG				nFetched = 0;

    hr = pFilter->EnumPins(&pEnum);
    if (FAILED(hr))
    {
		FAILED_TRACE;
        return FALSE;
    }

	vecInfo.clear();
	while (pEnum->Next(1, &pPin, &nFetched) == S_OK)
	{
		CPinInfo info;
		PIN_INFO pin_info;

		hr = pPin->QueryPinInfo(&pin_info);
		if (FAILED(hr))
		{
			pPin.Release();
			continue;
		}

		if (pin_info.pFilter != NULL)
		{
			pin_info.pFilter->Release();
			pin_info.pFilter = NULL;
		}

		info.bInput = pin_info.dir == PINDIR_INPUT ? TRUE : FALSE;
		pin_info.achName[MAX_PIN_NAME - 1] = L'\0';
		info.wsName = pin_info.achName;

		if (!GetPinCategory(pPin, info.wsCategory))
		{
			info.wsCategory = L"-";
		}

		if (!GetPinMediaType(pPin, info.wsMediaType))
		{
			info.wsMediaType = L"-";
		}

		pPin.Release();
		vecInfo.push_back(info);
	}

	if (vecInfo.size() <= 0)
	{
		return FALSE;
	}

	return TRUE;
}




BOOL CCaptureFilterInfo::GetVideoCaptureFormatInfo(IBaseFilter *pFilter, std::vector<CVideoCaptureFormatInfo> &vec)
{
	if (pFilter == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}

	HRESULT hr;
	CComPtr<ICaptureGraphBuilder2> pCGB2;
	CComPtr<IAMStreamConfig> pAMSC;
	int iCount = 0, iSize = 0;

	// ICaptureGraphBuilder2擾
	hr = CoCreateInstance(CLSID_CaptureGraphBuilder2, NULL, CLSCTX_INPROC, 
		IID_ICaptureGraphBuilder2, (LPVOID *)&pCGB2);

	if (FAILED(hr) || pCGB2 == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}


	hr = pCGB2->FindInterface(
		&PIN_CATEGORY_CAPTURE,	// Lv`[s
		&MEDIATYPE_Video,		// ׂẴfBA^Cv
		pFilter,				// Lv`tB^ւ̃|C^
		IID_IAMStreamConfig, (void**)&pAMSC);

	if (FAILED(hr) || pAMSC == NULL)
	{
		hr = pCGB2->FindInterface(
			&PIN_CATEGORY_PREVIEW,	// vr[s
			&MEDIATYPE_Video,		// ׂẴfBA^Cv
			pFilter,				// Lv`tB^ւ̃|C^
			IID_IAMStreamConfig, (void**)&pAMSC);

		if (FAILED(hr) || pAMSC == NULL)
		{
			FAILED_TRACE;
			return FALSE;
		}
	}


	hr = pAMSC->GetNumberOfCapabilities(&iCount, &iSize);
	if (FAILED(hr))
	{
		FAILED_TRACE;
		return FALSE;
	}

	LPBYTE pCap = new BYTE[iSize];

	if (pCap == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}

	vec.clear();

	for (int i = 0; i < iCount; i++)
	{
		CVideoCaptureFormatInfo vsc;
		AM_MEDIA_TYPE *pMT = NULL;

		hr = pAMSC->GetStreamCaps(i, &pMT, (BYTE*)pCap);
		if (SUCCEEDED(hr) && pMT != NULL && pCap != NULL)
		{
			if (pMT->majortype == MEDIATYPE_Video &&
				pMT->formattype == FORMAT_VideoInfo &&
				pMT->cbFormat >= sizeof(VIDEOINFOHEADER) &&
				pMT->pbFormat != NULL)
			{
				// \̃Rs[i|C^̓[Ŗ߂j
				vsc.mt = *pMT;
				vsc.mt.pbFormat = 0;
				vsc.mt.pUnk = 0;

				VIDEO_STREAM_CONFIG_CAPS *pssc = reinterpret_cast<VIDEO_STREAM_CONFIG_CAPS*>(pCap);
				vsc.ssc = *pssc;

				VIDEOINFOHEADER *pVih = reinterpret_cast<VIDEOINFOHEADER*>(pMT->pbFormat);
				vsc.vih = *pVih;

				MakeVideoFourCC(vsc.mt.subtype, vsc.vih.bmiHeader.biCompression, &(vsc.wsFourCC));
				MakeAnalogVideoStandard(vsc.ssc.VideoStandard, vsc.vecStandard);

				// AiOrfIKi̊l擾
				CComPtr<IAMAnalogVideoDecoder> pDecorder;
				pFilter->QueryInterface<IAMAnalogVideoDecoder>(&pDecorder);
				if (pDecorder != NULL)
				{
					long lElement;
					pDecorder->get_TVFormat(&lElement);
					vsc.lStandard = lElement;
				}
				else
				{
					vsc.lStandard = 0x00;
				}
				
				vec.push_back(vsc);
			}

			DeleteMediaType(pMT);
		}
	}

	delete [] pCap;

	if (vec.size() <= 0)
	{
		FAILED_TRACE;
		return FALSE;
	}

	return TRUE;
}


BOOL CCaptureFilterInfo::GetAudioCaptureFormatInfo(IBaseFilter *pFilter, std::vector<CAudioCaptureFormatInfo> &vec)
{
	if (pFilter == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}

	HRESULT hr;

	CComPtr<ICaptureGraphBuilder2> pCGB2;
	CComPtr<IAMStreamConfig> pAMSC;
	int iCount = 0, iSize = 0;

	// ICaptureGraphBuilder2擾
	hr = CoCreateInstance(CLSID_CaptureGraphBuilder2, NULL, CLSCTX_INPROC, 
		IID_ICaptureGraphBuilder2, (LPVOID *)&pCGB2);

	if (FAILED(hr) || pCGB2 == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}


	hr = pCGB2->FindInterface(
		&PIN_CATEGORY_CAPTURE,	// Lv`s^Cv
		&MEDIATYPE_Audio,		// I[fBIfBA^Cv
		pFilter,				// Lv`tB^ւ̃|C^
		IID_IAMStreamConfig, (void**)&pAMSC);

	if (FAILED(hr) || pAMSC == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}


	hr = pAMSC->GetNumberOfCapabilities(&iCount, &iSize);
	if (FAILED(hr))
	{
		FAILED_TRACE;
		return FALSE;
	}

	LPBYTE pCap = new BYTE[iSize];

	if (pCap == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}

	vec.clear();

	for (int i = 0; i < iCount; i++)
	{
		CAudioCaptureFormatInfo info;
		AM_MEDIA_TYPE *pMT = NULL;

		hr = pAMSC->GetStreamCaps(i, &pMT, (BYTE*)pCap);
		if (SUCCEEDED(hr) && pMT != NULL && pCap != NULL)
		{
			if (pMT->majortype == MEDIATYPE_Audio &&
				pMT->formattype == FORMAT_WaveFormatEx &&
				pMT->cbFormat >= sizeof(WAVEFORMATEX) &&
				pMT->pbFormat != NULL)
			{
				info.mt = *pMT;

				AUDIO_STREAM_CONFIG_CAPS *pssc = reinterpret_cast<AUDIO_STREAM_CONFIG_CAPS*>(pCap);
				info.ssc = *pssc;

				WAVEFORMATEX *pWav = reinterpret_cast<WAVEFORMATEX*>(pMT->pbFormat);
				info.wav = *pWav;

				MakeAudioFourCC(info.mt.subtype, &(info.wsFourCC));
				vec.push_back(info);
			}

			DeleteMediaType(pMT);
		}
	}

	delete [] pCap;

	if (vec.size() <= 0)
	{
		FAILED_TRACE;
		return FALSE;
	}

	return TRUE;
}


BOOL CCaptureFilterInfo::GetCrossbarInfo(IBaseFilter *pFilter, CCrossbarInfo &info1, CCrossbarInfo &info2)
{
	info1.vecInput.clear();
	info1.vecOutput.clear();
	info2.vecInput.clear();
	info2.vecOutput.clear();

	HRESULT hr;
	CComPtr<IGraphBuilder> pGB;
	CComPtr<ICaptureGraphBuilder2> pCGB2;

	if (!CreateBuilder(&pGB, &pCGB2))
	{
		FAILED_TRACE;
		return FALSE;
	}

	hr = pGB->AddFilter(pFilter, L"Video Capture");

	if (FAILED(hr))
	{
		FAILED_TRACE;
		return FALSE;
	}

	CComPtr<IAMCrossbar> pXBar1;
	CComPtr<IAMCrossbar> pXBar2;
	CComPtr<IBaseFilter> pCrossbar;


	hr = pCGB2->FindInterface(&LOOK_UPSTREAM_ONLY, NULL, pFilter,
		IID_IAMCrossbar, (void**)&pXBar1);

	if (SUCCEEDED(hr) && pXBar1 != NULL) 
	{
		GetCrossbarInfo(pXBar1, info1);

		hr = pXBar1->QueryInterface(IID_IBaseFilter, (void**)&pCrossbar);
		if (SUCCEEDED(hr) && pCrossbar != NULL) 
		{
			hr = pCGB2->FindInterface(&LOOK_UPSTREAM_ONLY, NULL, pCrossbar,
				IID_IAMCrossbar, (void**)&pXBar2);

			if (SUCCEEDED(hr) && pXBar2 != NULL)
			{
				GetCrossbarInfo(pXBar2, info2);
			}
		}
	}

	return TRUE;
}

LPCWSTR CCaptureFilterInfo::GetPhysicalPinName(long lType)
{
	switch (lType) 
	{
	case PhysConn_Video_Tuner:				return L"Video Tuner";
	case PhysConn_Video_Composite:			return L"Video Composite";
	case PhysConn_Video_SVideo:				return L"Video S-Video";
	case PhysConn_Video_RGB:				return L"Video RGB";
	case PhysConn_Video_YRYBY:				return L"Video YRYBY";
	case PhysConn_Video_SerialDigital:		return L"Video Serial Digital";
	case PhysConn_Video_ParallelDigital:	return L"Video Parallel Digital"; 
	case PhysConn_Video_SCSI:				return L"Video SCSI";
	case PhysConn_Video_AUX:				return L"Video AUX";
	case PhysConn_Video_1394:				return L"Video 1394";
	case PhysConn_Video_USB:				return L"Video USB";
	case PhysConn_Video_VideoDecoder:		return L"Video Decoder";
	case PhysConn_Video_VideoEncoder:		return L"Video Encoder";

	case PhysConn_Audio_Tuner:				return L"Audio Tuner";
	case PhysConn_Audio_Line:				return L"Audio Line";
	case PhysConn_Audio_Mic:				return L"Audio Microphone";
	case PhysConn_Audio_AESDigital:			return L"Audio AES/EBU Digital";
	case PhysConn_Audio_SPDIFDigital:		return L"Audio S/PDIF";
	case PhysConn_Audio_SCSI:				return L"Audio SCSI";
	case PhysConn_Audio_AUX:				return L"Audio AUX";
	case PhysConn_Audio_1394:				return L"Audio 1394";
	case PhysConn_Audio_USB:				return L"Audio USB";
	case PhysConn_Audio_AudioDecoder:		return L"Audio Decoder";

	default:								return L"Unknown Type";
	}    
}

//vG[`FbN
BOOL CCaptureFilterInfo::GetCrossbarInfo(IAMCrossbar *pXBar, CCrossbarInfo &info)
{
	if (pXBar == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}

	HRESULT hr;
	long lOutput = -1, lInput = -1;
	hr = pXBar->get_PinCounts(&lOutput, &lInput);

	CString csTmp;

    for (long i = 0; i < lOutput; i++)
    {
		CCrossbarOutputPinInfo out;
        long lRelated = -1, lType = -1, lRouted = -1;

        hr = pXBar->get_CrossbarPinInfo(FALSE, i, &lRelated, &lType);
        hr = pXBar->get_IsRoutedTo(i, &lRouted);

		csTmp.Format(L"%d:%s", i, GetPhysicalPinName(lType));

		out.lIndex = i;
		out.lRouted = lRouted;
		out.lRelated = lRelated;
		out.wsType = (LPCWSTR)csTmp;

		for (long j = 0; j < lInput; j++)
		{
			hr = pXBar->CanRoute(i, j);

			if (1/*hr == S_OK*/)//CanRoutelȂ
			{
				CCrossbarInputPinInfo in;
				long lRelated = -1, lType = -1, lRouted = -1;

				hr = pXBar->get_CrossbarPinInfo(TRUE, j, &lRelated, &lType);
				hr = pXBar->get_IsRoutedTo(j, &lRouted);


				csTmp.Format(L"%d:%s", j, GetPhysicalPinName(lType));

				in.lIndex = j;
				in.lRouted = lRouted;
				in.lRelated = lRelated;
				in.wsType = (LPCWSTR)csTmp;

				out.vecCanRoute.push_back(in);
			}
		}

		info.vecOutput.push_back(out);
    }

	for (long i = 0; i < lInput; i++)
	{
		CCrossbarInputPinInfo in;
		long lRelated = -1, lType = -1, lRouted = -1;

		hr = pXBar->get_CrossbarPinInfo(TRUE, i, &lRelated, &lType);
		hr = pXBar->get_IsRoutedTo(i, &lRouted);

#ifndef _DEBUG
		csTmp.Format(L"%s", GetPhysicalPinName(lType));
#else
		csTmp.Format(L"%d:%s", i, GetPhysicalPinName(lType));
#endif

		in.lIndex = i;
		in.lRouted = lRouted;
		in.lRelated = lRelated;
		in.wsType = (LPCWSTR)csTmp;

		info.vecInput.push_back(in);
	}

	return FALSE;
}

BOOL CCaptureFilterInfo::MakeAnalogVideoStandard(ULONG ulStandard, std::vector<CAnalogVideoStandardInfo> &vecStandard)
{
	vecStandard.clear();
	int nCount = (int)sizeof(ULONG) * 8; //rbg
	
	for (int i = 0; i < nCount; i++)
	{
		long lElement = 1 << i;
		ULONG ulElement = (ULONG)lElement;

		if (ulStandard & ulElement)
		{
			CAnalogVideoStandardInfo avs;
			switch (lElement)
			{
			//case AnalogVideo_None:
			case AnalogVideo_NTSC_M:
				{
					avs.wsElement = L"NTSC_M";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_NTSC_M_J:
				{
					avs.wsElement = L"NTSC_M_J";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_NTSC_433:
				{
					avs.wsElement = L"NTSC_433";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_PAL_B:
				{
					avs.wsElement = L"PAL_B";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_PAL_D:
				{
					avs.wsElement = L"PAL_D";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_PAL_G:
				{
					avs.wsElement = L"PAL_G";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_PAL_H:
				{
					avs.wsElement = L"PAL_H";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_PAL_I:
				{
					avs.wsElement = L"PAL_I";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_PAL_M:
				{
					avs.wsElement = L"PAL_M";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_PAL_N:
				{
					avs.wsElement = L"PAL_N";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_PAL_60:
				{
					avs.wsElement = L"PAL_60";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_SECAM_B:
				{
					avs.wsElement = L"SECAM_B";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_SECAM_D:
				{
					avs.wsElement = L"SECAM_D";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_SECAM_G:
				{
					avs.wsElement = L"SECAM_G";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_SECAM_H:
				{
					avs.wsElement = L"SECAM_H";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_SECAM_K:
				{
					avs.wsElement = L"SECAM_K";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_SECAM_K1:
				{
					avs.wsElement = L"SECAM_K1";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_SECAM_L:
				{
					avs.wsElement = L"SECAM_L";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_SECAM_L1:
				{
					avs.wsElement = L"SECAM_L1";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}
			case AnalogVideo_PAL_N_COMBO:
				{
					avs.wsElement = L"PAL_N_COMBO";
					avs.lElement = lElement;
					vecStandard.push_back(avs);
					break;
				}

			default:
				break;
			}
		}

	}
	return TRUE;
}



BOOL CCaptureFilterInfo::GetDirectShowFilterID(long lFilterCategory, std::vector<CDirectShowFilterID> &vecID)
{
	vecID.clear();

	HRESULT hr;

	CComPtr<ICreateDevEnum> pCreateDevEnum;
	CComPtr<IEnumMoniker> pEnumMoniker;
	CComPtr<IMoniker> pMoniker;
	ULONG nFetched = 0;

	// foCX񋓎q̏
	hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC_SERVER, 
		IID_ICreateDevEnum, (LPVOID *)&pCreateDevEnum);

	if (FAILED(hr) || pCreateDevEnum == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}


	switch (lFilterCategory)
	{
	case FILTER_CATEGORY_VIDEO_CAPTURE:
		hr = pCreateDevEnum->CreateClassEnumerator(CLSID_VideoInputDeviceCategory,
			&pEnumMoniker, 0);
		break;

	case FILTER_CATEGORY_AUDIO_CAPTURE:
		hr = pCreateDevEnum->CreateClassEnumerator(CLSID_AudioInputDeviceCategory,
			&pEnumMoniker, 0);
		break;

	default:
		hr = pCreateDevEnum->CreateClassEnumerator(CLSID_LegacyAmFilterCategory,
			&pEnumMoniker, 0);
		break;
	}

	if (FAILED(hr) || pEnumMoniker == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}


	// MonikerZbg
	hr = pEnumMoniker->Reset();

	if (FAILED(hr))
	{
		FAILED_TRACE;
		return FALSE;
	}

	while (pEnumMoniker->Next(1, &pMoniker, &nFetched) == S_OK)
	{
		if (pMoniker == NULL)
		{
			FAILED_TRACE;
			continue;
		}

		CDirectShowFilterID id;

		id.lFilterCategory = lFilterCategory;

		CComPtr<IPropertyBag> pPropertyBag;
		VARIANT var;

		hr = pMoniker->BindToStorage(0, 0, IID_IPropertyBag,
			(void **)&pPropertyBag);

		if (FAILED(hr) || pPropertyBag == NULL)
		{
			pMoniker.Release();
			FAILED_TRACE;
			continue;
		}

		// FrienlyName : foCX̖O
		// vpeBׂ͂ẴfoCXŗpłB
		// lǂނƂłfoCXi[B
		var.vt = VT_BSTR;
		hr = pPropertyBag->Read(L"FriendlyName", &var, 0);

		if (FAILED(hr))
		{
			id.wsFriendlyName = L"";
		}
		else
		{
			id.wsFriendlyName = var.bstrVal;
		}

		VariantClear(&var);

		if (id.wsFriendlyName == L"")
		{
			pMoniker.Release();
			FAILED_TRACE;
			continue;
		}


		// Description : foCX̐
		// vpeB DV  D-VHS/MPEG ̃JR[_ foCXł̂ݗpłB
		// ڍׂɂẮAuMSDV hCovƁuMSTape hCovQƂ邱ƁB
		// płꍇAFriendlyName vpeBڂfoCX̐i[B
		// ʏAx_B
		var.vt = VT_BSTR;
		hr = pPropertyBag->Read(L"Description", &var, 0);

		if (FAILED(hr))
		{
			id.wsDescription = L"";
		}
		else
		{
			id.wsDescription = var.bstrVal;
		}

		VariantClear(&var);


		// DevicePath : ӂ̕
		// vpeB͐lǂނƂł镶ł͂ȂAfoCXƂɈӂł邱Ƃۏ؂ĂB
		// ̃vpeBgAfoCX̓ꃂf̕CX^XʂłB
		var.vt = VT_BSTR;
		hr = pPropertyBag->Read(L"DevicePath", &var, 0);

		if (FAILED(hr))
		{
			id.wsDevicePath = L"";
		}
		else
		{
			id.wsDevicePath = var.bstrVal;
		}

		VariantClear(&var);

		vecID.push_back(id);

		// Moniker邽߂Ɏ蓮Release
		pMoniker.Release();
	}

	return TRUE;
}


BOOL CCaptureFilterInfo::GetDirectShowFilter(const CDirectShowFilterID &id, IBaseFilter **ppFilter)
{
	if (ppFilter == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}

	HRESULT hr;

	CComPtr<ICreateDevEnum> pCreateDevEnum;
	CComPtr<IEnumMoniker> pEnumMoniker;
	CComPtr<IMoniker> pMoniker;
	ULONG nFetched = 0;
	BOOL bResult = FALSE;

	hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC_SERVER, 
		IID_ICreateDevEnum, (LPVOID *)&pCreateDevEnum);
	if (FAILED(hr) || pCreateDevEnum == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}

	switch (id.lFilterCategory)
	{
	case FILTER_CATEGORY_VIDEO_CAPTURE:
		hr = pCreateDevEnum->CreateClassEnumerator(CLSID_VideoInputDeviceCategory,
			&pEnumMoniker, 0);
		break;

	case FILTER_CATEGORY_AUDIO_CAPTURE:
		hr = pCreateDevEnum->CreateClassEnumerator(CLSID_AudioInputDeviceCategory,
			&pEnumMoniker, 0);
		break;

	default:
		hr = pCreateDevEnum->CreateClassEnumerator(CLSID_LegacyAmFilterCategory,
			&pEnumMoniker, CDEF_DEVMON_FILTER|CDEF_DEVMON_DMO);
		break;
	}

	if (FAILED(hr) || pEnumMoniker == NULL)
	{
		FAILED_TRACE;
		return FALSE;
	}

	hr = pEnumMoniker->Reset();
	if (FAILED(hr))
	{
		FAILED_TRACE;
		return FALSE;
	}


	while (pEnumMoniker->Next(1, &pMoniker, &nFetched) == S_OK)
	{
		if (pMoniker == NULL)
		{
			FAILED_TRACE;
			continue;
		}

		CComPtr<IPropertyBag> pPropertyBag;
		VARIANT var;
		std::wstring wsFriendlyName;
		std::wstring wsDevicePath;

		hr = pMoniker->BindToStorage(0, 0, IID_IPropertyBag,
			(void **)&pPropertyBag);

		if (FAILED(hr) || pPropertyBag == NULL)
		{
			pMoniker.Release();
			FAILED_TRACE;
			continue;
		}

		// FrienlyName : foCX̖O
		// vpeBׂ͂ẴfoCXŗpłB
		// lǂނƂłfoCXi[B
		var.vt = VT_BSTR;
		hr = pPropertyBag->Read(L"FriendlyName", &var, 0);

		if (FAILED(hr))
		{
			wsFriendlyName = L"";
		}
		else
		{
			wsFriendlyName = var.bstrVal;
		}

		VariantClear(&var);

		if (wsFriendlyName == L"")
		{
			pMoniker.Release();
			FAILED_TRACE;
			continue;
		}

		// DevicePath : ӂ̕
		// vpeB͐lǂނƂł镶ł͂ȂAfoCXƂɈӂł邱Ƃۏ؂ĂB
		// ̃vpeBgAfoCX̓ꃂf̕CX^XʂłB
		var.vt = VT_BSTR;
		hr = pPropertyBag->Read(L"DevicePath", &var, 0);

		if (FAILED(hr))
		{
			wsDevicePath = L"";
		}
		else
		{
			wsDevicePath = var.bstrVal;
		}

		VariantClear(&var);


		if (id.wsFriendlyName == wsFriendlyName  &&
			id.wsDevicePath == wsDevicePath)
		{
			pMoniker->BindToObject(0, 0, IID_IBaseFilter, (void**)ppFilter);
			pMoniker.Release();
			bResult = TRUE;
			break;
		}
		
		pMoniker.Release();
	}
	
	return bResult;
}

BOOL CCaptureFilterInfo::GetDirectShowFilterInfo(const CDirectShowFilterID &id, CDirectShowFilterInfo &info)
{
	if (info.bIsLoaded)
	{
		return TRUE;
	}

	CComPtr<IBaseFilter> pFilter;
	if (!GetDirectShowFilter(id, &pFilter))
	{
		FAILED_TRACE;
		return FALSE;
	}

	info.wsFriendlyName = id.wsFriendlyName;
	info.wsDevicePath = id.wsDevicePath;
	info.wsDescription = id.wsDescription;

	if (!GetPinInfo(pFilter, info.vecPin))
	{
		FAILED_TRACE;
		return FALSE;
	}

	return TRUE;
}

BOOL CCaptureFilterInfo::GetVideoCaptureFilterInfo(const CDirectShowFilterID &id, CVideoCaptureFilterInfo &info)
{
	CComPtr<IBaseFilter> pFilter;
	if (!GetDirectShowFilter(id, &pFilter))
	{
		FAILED_TRACE;
		return FALSE;
	}

	if (!GetPinInfo(pFilter, info.vecPin))
	{
		FAILED_TRACE;
		return FALSE;
	}

	// tH[}bg
	// tH[}bgĂڑł
	GetVideoCaptureFormatInfo(pFilter, info.vecFormat);

	// NXo[
	// NXo[͂Ȃꍇ̂ł̂܂
	GetCrossbarInfo(pFilter, info.Xbar1, info.Xbar2);

	return TRUE;
}

BOOL CCaptureFilterInfo::GetAudioCaptureFilterInfo(const CDirectShowFilterID &id, CAudioCaptureFilterInfo &info)
{
	CComPtr<IBaseFilter> pFilter;
	if (!GetDirectShowFilter(id, &pFilter))
	{
		FAILED_TRACE;
		return FALSE;
	}

	if (!GetPinInfo(pFilter, info.vecPin))
	{
		FAILED_TRACE;
		return FALSE;
	}

	if (!HasAudioCapturePin(info.vecPin))
	{
		FAILED_TRACE;
		return FALSE;
	}

	// tH[}bg
	// tH[}bgĂڑł
	GetAudioCaptureFormatInfo(pFilter, info.vecFormat);

	return TRUE;
}
