#pragma once

#include "MyTransform.h"
#include "CaptureFilterInfo.h"

#include "AVIMuxFilter.h"

#define WM_GRAPH_NOTIFY		(WM_APP + 20)


typedef enum _VIDEO_RENDERER {
	VIDEO_RENDERER_VMR7 = 0,
	VIDEO_RENDERER_VMR7_OFFSCREEN = 1,
	VIDEO_RENDERER_VMR9 = 2,
	VIDEO_RENDERER_EVR = 3,
	VIDEO_RENDERER_EVR_CUSTOM = 4,
} VIDEO_RENDERER;

typedef enum _AUDIO_RENDERER {
	AUDIO_RENDERER_DIRECTSOUND = 0,
	AUDIO_RENDERER_WAVEOUT = 1,
} AUDIO_RENDERER;

typedef enum _ASPECT_RATIO {
	ASPECT_RATIO_INPUT = 0,
	ASPECT_RATIO_4_3,
	ASPECT_RATIO_16_9,
	ASPECT_RATIO_40_33,
	ASPECT_RATIO_10_11,
	ASPECT_RATIO_CUSTOM,
	ASPECT_RATIO_WINDOW = 100,
} ASPECT_RATIO;

// vZXDx
typedef enum _PROCESS_PRIORITY {
	PROCESS_PRIORITY_NORMAL = 0,
	PROCESS_PRIORITY_BELOW_NOMAL,
	PROCESS_PRIORITY_IDLE,
	PROCESS_PRIORITY_ABOVE_NORMAL,
	PROCESS_PRIORITY_HIGH,
	PROCESS_PRIORITY_REALTIME,
} PROCESS_PRIORITY;


// VideoProcAmp
class CVideoProcAmp
{
public:
	CVideoProcAmp() :
		lProperty(LONG_MIN),
		lMin(LONG_MIN),
		lMax(LONG_MIN),
		lSteppingDelta(LONG_MIN),
		lDefault(LONG_MIN),
		lCapsFlags(LONG_MIN),
		lValue(LONG_MIN),
		lFlags(LONG_MIN)
	{}

	long lProperty;
	long lMin;
	long lMax;
	long lSteppingDelta;
	long lDefault;
	long lCapsFlags;

	long lValue;
	long lFlags;
};

extern const VideoProcAmpProperty g_VideoProcAmpProp[];

class CVideoProcAmpConfig {
public:
	CVideoProcAmpConfig() :
		lValue(LONG_MIN),
		lFlags(LONG_MIN)
	{}
	long lValue;
	long lFlags;
};

//--------------------------------
class CVMR9ProcAmpRange {
public:
	CVMR9ProcAmpRange()
		: nDefaultValue(0)
		, nMaxValue(0)
		, nMinValue(0)
		, nStepSize(0)
	{}

	int nDefaultValue;
	int nMaxValue;
	int nMinValue;
	int nStepSize;
};

class CVMR9ProcAmpRanges {
public:
	CVMR9ProcAmpRange Brightness;
	CVMR9ProcAmpRange Contrast;
	CVMR9ProcAmpRange Hue;
	CVMR9ProcAmpRange Saturation;
};

class CVMR9ProcAmpConfig {
public:
	CVMR9ProcAmpConfig()
		: nBrightness(INT_MIN)
		, nContrast(INT_MIN)
		, nHue(INT_MIN)
		, nSaturation(INT_MIN)
	{}

	int nBrightness;
	int nContrast;
	int nHue;
	int nSaturation;
};


//--------------------------------
class CEVRProcAmpRange {
public:
	CEVRProcAmpRange()
		: nDefaultValue(0)
		, nMaxValue(0)
		, nMinValue(0)
		, nStepSize(0)
	{}

	int nDefaultValue;
	int nMaxValue;
	int nMinValue;
	int nStepSize;
};

class CEVRProcAmpRanges {
public:
	CEVRProcAmpRange Brightness;
	CEVRProcAmpRange Contrast;
	CEVRProcAmpRange Hue;
	CEVRProcAmpRange Saturation;
};

class CEVRProcAmpConfig {
public:
	CEVRProcAmpConfig()
		: nBrightness(INT_MIN)
		, nContrast(INT_MIN)
		, nHue(INT_MIN)
		, nSaturation(INT_MIN)
	{}

	int nBrightness;
	int nContrast;
	int nHue;
	int nSaturation;
};


//////////////////////////////////////////////////////////////////////////////////////
// o̓tH[}bgݒpNXQ

// tH[}bg̓e͏d\̂ŃtH[}bgԍŎw肷


class CVideoCaptureFormat {
public:
	CVideoCaptureFormat()
		: nIndex(0)
		, biWidth(0)
		, biHeight(0)
		, AvgTimePerFrame(0)
		, lStandard(0)
	{}

	int nIndex; // tH[}bgԍ
	LONG biWidth; // (BITMAPINFOHEADER)
	LONG biHeight; // (BITMAPINFOHEADER)
	REFERENCE_TIME AvgTimePerFrame; // FPS (VIDEOINFOHEADER)

	long lStandard; // AiOrfIKĩrbgtO
};

class CAudioCaptureFormat {
public:
	CAudioCaptureFormat()
		: nIndex(0)
		, nChannels(0)
		, wBitsPerSample(0)
		, nSamplesPerSec(0)
	{}

	int nIndex; // tH[}bgԍ
    WORD        nChannels; // `l
    WORD        wBitsPerSample; // BPS
    DWORD       nSamplesPerSec; // g
};

class CCrossbar {
public:
	CCrossbar()
		: lOutput(-1)
		, lInput(-1)
	{}
	long lOutput;
	long lInput;
};

class CVideoCaptureFilter
{
public:
	std::wstring wsFriendlyName;
	std::wstring wsDescription;
	std::wstring wsDevicePath;

	std::vector<CCrossbar> vecCrossbar;

	CVideoCaptureFormat Format;
};

class CAudioCaptureFilter
{
public:
	CAudioCaptureFilter()
		: bBufferSize(FALSE)
		, nBufferSize(50)
	{}
	std::wstring wsFriendlyName;
	std::wstring wsDescription;
	std::wstring wsDevicePath;

	BOOL bBufferSize;
	int nBufferSize;

	CAudioCaptureFormat Format;
};


class CPreviewConfig {
public:
	CPreviewConfig()
		: bVideoPreviewPin(TRUE)
		, bAudioPreviewPin(TRUE)
		, lVideoRenderer(VIDEO_RENDERER_EVR_CUSTOM)
		, lAudioRenderer(AUDIO_RENDERER_DIRECTSOUND)
		, bTransform(TRUE)
		, lTransformCopy(TRANSFORM_COPY_NONE)
		, lTransformFrame(TRANSFORM_FRAME_NONE)
		, lTransformFieldOrder(TRANSFORM_FIELD_ORDER_ODD)
		, lTransformRotation(TRANSFORM_ROTATION_NONE)
		, bTransformFlipHorizontal(FALSE)
		, bTransformFlipVertical(FALSE)
		, bTransformDoubleHeight(FALSE)
		, lTransformYCExpand(TRANSFORM_YCEXPAND_NONE)
		, nAudioVolume(100)
		, bAudioMute(FALSE)
		, lAspectRatio(ASPECT_RATIO_INPUT)
		, nAspectX(4) // l4:3͏EBhETCY4:3Ȃ
		, nAspectY(3)
		, lProcessPriority(PROCESS_PRIORITY_ABOVE_NORMAL)
		, rcCropping(RECT())
		, lResizer(d3dutl::RESIZER_DEFAULT)
	{
		OSVERSIONINFO info;
		info.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		if (GetVersionEx(&info))
		{
			// dwMajorVersion == 6 Vista
			if (info.dwMajorVersion < 6)
			{
				lVideoRenderer = VIDEO_RENDERER_VMR7;
			}
		}
	}
	BOOL bVideoPreviewPin;
	BOOL bAudioPreviewPin;
	long lVideoRenderer;
	long lAudioRenderer;
	BOOL bTransform;
	long lTransformCopy;
	long lTransformFrame;
	long lTransformFieldOrder;
	long lTransformRotation;
	BOOL bTransformFlipHorizontal;
	BOOL bTransformFlipVertical;
	BOOL bTransformDoubleHeight;
	long lTransformYCExpand;
	int nAudioVolume;
	BOOL bAudioMute;
	long lAspectRatio;
	int nAspectX;
	int nAspectY;
	long lProcessPriority;
	RECT rcCropping;
	int lResizer;
};



class CFilterItem {
public:
	CFilterItem() : nInput(-1), nOutput(-1), bCheck(TRUE) {}
	~CFilterItem() {}

	std::wstring wsFriendlyName;
	std::wstring wsDescription;
	std::wstring wsDevicePath;
	int nInput;
	int nOutput;
	BOOL bCheck;
};

class CFilterConfig {
public:
	std::vector<CFilterItem> vecVideo;
	std::vector<CFilterItem> vecAudio;
};


class CCaptureGraphProfile {
public:
	std::wstring wsName;
	CVideoCaptureFilter Video;
	CAudioCaptureFilter Audio;
	CFilterConfig Filter;
	CPreviewConfig Preview;

	CVideoProcAmpConfig VideoProcAmp[10];
	CVMR9ProcAmpConfig VMR9ProcAmp;
	CEVRProcAmpConfig EVRProcAmp;
};

class CFilterGraph {
public:
	CFilterGraph() : bHasProperty(FALSE) {}
	~CFilterGraph() {}

	std::wstring wsFriendlyName;
	BOOL bHasProperty;
};



class CCaptureGraphBuilder
{
public:
	CCaptureGraphBuilder(void);
	~CCaptureGraphBuilder(void);

	BOOL Init(void);
	void UnInit(void);

	BOOL CreateBuilder(IGraphBuilder** ppGB, ICaptureGraphBuilder2** ppCGB2);

	BOOL GetFilter(long lFilterCategory, LPCWSTR pwszFriendlyName, LPCWSTR pwszDevicePath, IBaseFilter **ppFilter);

	BOOL ConnectFilter(IGraphBuilder *pGB, ICaptureGraphBuilder2 *pCGB2, IBaseFilter *pOutput, int nOutput, IBaseFilter *pInput, int nInput);

	BOOL ConnectHelper(BOOL bVideo, BOOL bPreview,  std::vector<CFilterItem> &vecFilter,
		IGraphBuilder *pGB, ICaptureGraphBuilder2 *pCGB2,
		IBaseFilter *pCapture, IBaseFilter *pSmartTee, IBaseFilter *pRenderer);

	BOOL ConnectHelperX(BOOL bVideo, BOOL bPreview,
		IGraphBuilder *pGB, ICaptureGraphBuilder2 *pCGB2,
		IBaseFilter *pCapture, IBaseFilter *pSmartTee, IBaseFilter *pTransform, IBaseFilter *pRenderer);

	BOOL ConnectMux(BOOL bVideo, BOOL bPreview,
		IGraphBuilder *pGB, ICaptureGraphBuilder2 *pCGB2, IBaseFilter *pTee);


	BOOL SetVideoCaptureFormat(IBaseFilter *pFilter, CVideoCaptureFormat &format);
	BOOL SetAudioCaptureFormat(IBaseFilter *pFilter, CAudioCaptureFormat &format);

	BOOL Preview(HWND hWnd, CCaptureGraphProfile &profile);

	BOOL GetFilterGraph(IGraphBuilder *pGB, std::vector<CFilterGraph> &vecGraph);
	BOOL GetFilterGraph(std::vector<CFilterGraph> &vecGraph);
	BOOL ShowFilterProperty(int nIndex);

	BOOL SetTransformCopy(long _trans_copy);
	BOOL SetTransformFrame(long _trans_frame);
	BOOL SetTransformFieldOrder(long _trans_order);
	BOOL SetTransformFlipVertical(BOOL _trans_flip);
	BOOL SetTransformYCExpand(long _trans_ycexpand);

	BOOL SetResizer(int iResizer);

	BOOL Run(void);
	BOOL Pause(void);
	BOOL Stop(void);
	BOOL IsRun(void);
	BOOL IsVideoRenderer(void);
	BOOL IsAudioRenderer(void);

	//BOOL GraphClear(void);

	BOOL GetQualProp(int *piAvgFrameRate, int *piDropFrames);

	BOOL GetNativeVideoSize(LPLONG lpWidth, LPLONG lpHeight);
	BOOL RepaintVideo(void);
	BOOL DisplayModeChanged(void);
	BOOL SetVideoPosition(void);
	BOOL ConvertAspectRatio(long lWidth, long lHeight, long *lpAspectCX, long *lpAspectCY, BOOL bAuto = TRUE, BOOL bApplyWidth = TRUE);


	BOOL GetCurrentImage(BITMAPINFO *pBmpInfo, BYTE **ppDIB);
	//BOOL GetCurrentImage(BYTE **ppDIB, DWORD *pdwDIBSize);
	

	void SetCroppingRect(RECT &rc);
	void SetAspectRatio(int nAspectX, int nAspectY);
	void SetVideoSize(long lVideoWidth, long lVideoHeight);

	BOOL SetAudioVolume(int nVolume); // 0~100%

	BOOL GetEvent(long *lEventCode, LONG_PTR *lParam1, LONG_PTR *lParam2);

	BOOL GetVideoProcAmp(CVideoProcAmp *pAmp);
	BOOL SetVideoProcAmp(long Property, long lValue, long Flags);

	BOOL GetVideoProcAmpConfig(CVideoProcAmpConfig *pAmp);
	BOOL SetVideoProcAmpConfig(CVideoProcAmpConfig *pAmp);

	BOOL GetVMR9ProcAmpRange(CVMR9ProcAmpRanges &amp);
	BOOL GetVMR9ProcAmp(CVMR9ProcAmpConfig &amp);
	BOOL SetVMR9ProcAmp(const CVMR9ProcAmpConfig &amp);

	BOOL GetEVRProcAmpRange(CEVRProcAmpRanges &amp);
	BOOL GetEVRProcAmp(CEVRProcAmpConfig &amp);
	BOOL SetEVRProcAmp(const CEVRProcAmpConfig &amp);


	HRESULT AddToRot(IUnknown *pUnkGraph) 
	{
		HRESULT hr = E_NOTIMPL;

#ifdef _DEBUG

		if (m_dwRegister != 0)
		{
			RemoveFromRot();
		}

		CComPtr<IMoniker> pMoniker;
		CComPtr<IRunningObjectTable> pROT;

		if (FAILED(GetRunningObjectTable(0, &pROT)))
		{
			return E_FAIL;
		}

		WCHAR wsz[256];
		wsprintfW(wsz, L"FilterGraph %08p pid %08x", (DWORD_PTR)pUnkGraph, GetCurrentProcessId());

		hr = CreateItemMoniker(L"!", wsz, &pMoniker);
		if (SUCCEEDED(hr))
		{
			hr = pROT->Register(0, pUnkGraph, pMoniker, &m_dwRegister);
		}

#endif

		return hr;
	}

	void RemoveFromRot(void)
	{
#ifdef _DEBUG

		if (m_dwRegister == 0)
		{
			return;
		}

		CComPtr<IRunningObjectTable> pROT;
		if (SUCCEEDED(GetRunningObjectTable(0, &pROT)))
		{
			pROT->Revoke(m_dwRegister);
		}

#endif
	}

	void ResetLog(void);
	void AddLog(LPCWSTR pwszLog, ...);
	LPCWSTR GetLog(void);



public:
	std::vector<CFilterGraph> m_vecGraph; // tB^Ot̏

private:
	std::wstring					m_wsLog;
	BOOL							m_bCoInitialize;

	FOURCC							m_bVideoFormatFourCC; //SetVideoCaptureFormatŎ擾BVMR Mixxing Mode̔ɕKvB

	CComPtr<IGraphBuilder>			m_pGB;
	CComPtr<ICaptureGraphBuilder2>	m_pCGB2;

	CComPtr<IMediaControl>			m_pMC;
	CComPtr<IMediaEventEx>			m_pME;

	CComPtr<IBaseFilter>			m_pVC; // Capture
	CComPtr<IBaseFilter>			m_pAC;

	CComPtr<IBaseFilter>			m_pAR; // Renderer
	CComPtr<IBaseFilter>			m_pVR;

	CComPtr<IBaseFilter>			m_pVS; // SmartTee
	CComPtr<IBaseFilter>			m_pAS;

	CComPtr<IBaseFilter>			m_pTF; // Transform
	CComPtr<IBaseFilter>			m_pMux; // Mux

	CComPtr<ICustomRenderProp>		m_pProp; // EVRCustomݒ


	HWND m_hWnd;
	// rfI_
	long m_lVideoRenderer;
	// EBhEݒ
	int	m_nWndAspectX; // AXyNg
	int	m_nWndAspectY;

	// rfITCY
	long m_lVideoWidth;
	long m_lVideoHeight;

	// NbsOTCY
	CRect m_rcCrop;

#ifdef _DEBUG
	DWORD							m_dwRegister;
#endif
};



