#include "stdafx.h"
#include "Config.h"
#include "PeCaTV2.h"


//	BOOL -> bool ϊ
template<typename T>
static bool Bool(T x)
{
	return x ? true : false;
}



//--------------------------------------------------------------------------------
//	WriteConfigINI
//	ݒt@C
//--------------------------------------------------------------------------------
void CConfig::WriteConfigXML()
{
	XmlParser xml;
	auto root = xml.put(L"root");
	root.put(L"version", CONFIG_FILE_VERSION);
	root.put(L"initial", false);
	
	auto x = root.put(L"windowPlacement");
	x.put(L"length", m_stWndPlacement.length);
	x.put(L"flags", m_stWndPlacement.flags);
	x.put(L"showCmd", m_stWndPlacement.showCmd);

	x = root.put(L"windowPlacement.ptMinPosition");
	x.put(L"x", m_stWndPlacement.ptMinPosition.x);
	x.put(L"y", m_stWndPlacement.ptMinPosition.y);

	x = root.put(L"windowPlacement.ptMaxPosition");
	x.put(L"x", m_stWndPlacement.ptMaxPosition.x);
	x.put(L"y", m_stWndPlacement.ptMaxPosition.y);

	x = root.put(L"windowPlacement.rcNormalPosition");
	x.put(L"left", m_stWndPlacement.rcNormalPosition.left);
	x.put(L"top", m_stWndPlacement.rcNormalPosition.top);
	x.put(L"right", m_stWndPlacement.rcNormalPosition.right);
	x.put(L"bottom", m_stWndPlacement.rcNormalPosition.bottom);

	x = root.put(L"window");
	x.put(L"topMost", Bool(m_bTopMost));
	x.put(L"title", Bool(m_bTitleBar));
	x.put(L"frame", Bool(m_bWindowFrame));
	x.put(L"aspectRatio", Bool(m_bWindowAspectRatio));
	x.put(L"snap", Bool(m_bWindowSnap));
	x.put(L"drag", Bool(m_bWindowDrag));

	x = root.put(L"control");
	x.put(L"screenSaver", Bool(m_bScreenSaverStop));
	x.put(L"monitorPower", Bool(m_bMonitorPowerOffStop));
	x.put(L"systemStandby", Bool(m_bSystemStandbyStop));

	x = root.put(L"size");
	for (auto it = m_vecWindowSize.begin(); it != m_vecWindowSize.end(); ++it)
	{
		auto item = x.add(L"item");
		item.put(L"enable", Bool(it->enable));
		item.put(L"width", it->width);
		item.put(L"height", it->height);
	}

	x = root.put(L"zoom");
	for (auto it = m_vecWindowZoom.begin(); it != m_vecWindowZoom.end(); ++it)
	{
		auto item = x.add(L"item");
		item.put(L"enable", Bool(it->enable));
		item.put(L"zoom", it->zoom);
	}

	x = root.put(L"crop");
	for (auto it = m_vecCrop.begin(); it != m_vecCrop.end(); ++it)
	{
		auto item = x.add(L"item");
		item.put(L"left", it->left);
		item.put(L"top", it->top);
		item.put(L"right", it->right);
		item.put(L"bottom", it->bottom);
	}

	x = root.put(L"snapshot");
	x.put(L"folder", m_wsSnapShotFolder);
	x.put(L"type", m_lSnapShotType);
	x.put(L"size", m_nSnapShotVideoSize);
	x.put(L"crop", Bool(m_bSnapShotCrop));
	x.put(L"jpegQuality", m_nSnapShotJpegQuality);

	x = root.put(L"option");
	x.put(L"lastTab", m_nOptionLastTab);


	x = root.put(L"shortcut");
	
	for (auto it = m_vecShortCut.begin(); it != m_vecShortCut.end(); ++it)
	{
		auto item = x.add(L"item");
		item.put(L"messageID", it->dwMessageID);
		item.put(L"keyModify", it->dwKeyModify);
		item.put(L"keyCode", it->dwKeyCode);
		item.put(L"mouseModify", it->dwMouseModify);
		item.put(L"mouseCode", it->dwMouseCode);
	}
	
	xml.write(m_wsConfigFilePathXML);
}


//--------------------------------------------------------------------------------
//	WriteProfileXML
//	ݒt@C
//--------------------------------------------------------------------------------
void CConfig::WriteProfileXML()
{
	XmlParser xml;
	auto root = xml.put(L"root");
	root.put(L"version", CONFIG_FILE_VERSION);

	auto x = root.put(L"option");
	x.put(L"lastTab", m_nProfileLastTab);
	x.put(L"preview", Bool(m_bProfilePreview));

	x = root.put(L"profile");
	x.put(L"selected", m_nProfileSelected);
	for (auto it = m_vecProfile.begin(); it != m_vecProfile.end(); ++it)
	{
		auto item = x.add(L"item");
		item.put(L"name", it->wsName);

		WriteProfileItemXML(item, *it);
	}

	xml.write(m_wsProfileFilePathXML);
}

void CConfig::WriteProfileItemXML(XmlParser& root, CCaptureGraphProfile& profile)
{
	auto x = root.put(L"video");
	x.put(L"name", profile.Video.wsFriendlyName);
	x.put(L"path", profile.Video.wsDevicePath);
	x.put(L"desc", profile.Video.wsDescription);

	x = root.put(L"video.format");
	x.put(L"index", profile.Video.Format.nIndex);
	x.put(L"width", profile.Video.Format.biWidth);
	x.put(L"height", profile.Video.Format.biHeight);
	x.put(L"avgTimePerFrame", profile.Video.Format.AvgTimePerFrame);
	x.put(L"standard", profile.Video.Format.lStandard);

	WriteProfileCrossbarXML(x, profile.Video.vecCrossbar);

	x = root.put(L"audio");
	x.put(L"name", profile.Audio.wsFriendlyName);
	x.put(L"path", profile.Audio.wsDevicePath);
	x.put(L"desc", profile.Audio.wsDescription);

	x = root.put(L"audio.buffer");
	x.put(L"enable", Bool(profile.Audio.bBufferSize));
	x.put(L"size", profile.Audio.nBufferSize);

	x = root.put(L"audio.format");
	x.put(L"index", profile.Audio.Format.nIndex);
	x.put(L"channels", profile.Audio.Format.nChannels);
	x.put(L"bitsPerSample", profile.Audio.Format.wBitsPerSample);
	x.put(L"samplesPerSec", profile.Audio.Format.nSamplesPerSec);

	x = root.put(L"filter.video");
	WriteProfileFilterXML(x, profile.Filter.vecVideo);

	x = root.put(L"filter.audio");
	WriteProfileFilterXML(x, profile.Filter.vecAudio);


	x = root.put(L"preview");
	x.put(L"processPriority", profile.Preview.lProcessPriority);

	x = root.put(L"preview.video");
	x.put(L"renderder", profile.Preview.lVideoRenderer);
	x.put(L"sync", !Bool(profile.Preview.bVideoPreviewPin)); // !۔]
	x.put(L"aspectRatio", profile.Preview.lAspectRatio);
	x.put(L"aspectX", profile.Preview.nAspectX);
	x.put(L"aspectY", profile.Preview.nAspectY);
	x.put(L"resizer", profile.Preview.lResizer);

	x = root.put(L"preview.audio");
	x.put(L"renderder", profile.Preview.lAudioRenderer);
	x.put(L"sync", !Bool(profile.Preview.bAudioPreviewPin)); // !۔]
	x.put(L"volume", profile.Preview.nAudioVolume);
	x.put(L"mute", profile.Preview.bAudioMute);

	x = root.put(L"preview.transform");
	x.put(L"enable", Bool(profile.Preview.bTransform)); 
	x.put(L"copy", profile.Preview.lTransformCopy);
	x.put(L"frame", profile.Preview.lTransformFrame);
	x.put(L"fieldOrder", profile.Preview.lTransformFieldOrder);
	x.put(L"rotation", profile.Preview.lTransformRotation);
	x.put(L"flipHorizontal", Bool(profile.Preview.bTransformFlipHorizontal));
	x.put(L"flipVertical", Bool(profile.Preview.bTransformFlipVertical));
	x.put(L"doubleHeight", Bool(profile.Preview.bTransformDoubleHeight));
	x.put(L"ycExpand", profile.Preview.lTransformYCExpand);

	x = root.put(L"preview.crop");
	x.put(L"left", profile.Preview.rcCropping.left);
	x.put(L"top", profile.Preview.rcCropping.top);
	x.put(L"right", profile.Preview.rcCropping.right);
	x.put(L"bottom", profile.Preview.rcCropping.bottom);

	x = root.put(L"preview.videoProcAmp");
	WriteProfileVideoProcAmpXML(x, profile.VideoProcAmp);

	x = root.put(L"preview.vmr9ProcAmp");
	x.put(L"brightness", profile.VMR9ProcAmp.nBrightness);
	x.put(L"contrast", profile.VMR9ProcAmp.nContrast);
	x.put(L"hue", profile.VMR9ProcAmp.nHue);
	x.put(L"saturation", profile.VMR9ProcAmp.nSaturation);

	x = root.put(L"preview.evrProcAmp");
	x.put(L"brightness", profile.EVRProcAmp.nBrightness);
	x.put(L"contrast", profile.EVRProcAmp.nContrast);
	x.put(L"hue", profile.EVRProcAmp.nHue);
	x.put(L"saturation", profile.EVRProcAmp.nSaturation);
}

void CConfig::WriteProfileCrossbarXML(XmlParser& root, std::vector<CCrossbar>& crossbar)
{
	auto x = root.put(L"crossbar");
	for (auto it = crossbar.begin(); it != crossbar.end(); ++it)
	{
		auto item = x.add(L"item");
		item.put(L"input", it->lInput);
		item.put(L"output", it->lOutput);
	}
}

void CConfig::WriteProfileFilterXML(XmlParser& root, std::vector<CFilterItem>& filter)
{
	for (auto it = filter.begin(); it != filter.end(); ++it)
	{
		auto item = root.add(L"item");
		item.put(L"enable", Bool(it->bCheck));
		item.put(L"name", it->wsFriendlyName);
		item.put(L"path", it->wsDevicePath);
		item.put(L"desc", it->wsDescription);
		item.put(L"input", it->nInput);
		item.put(L"output", it->nOutput);
	}
}

extern const wchar_t* videoProcAmpKey[10];
void CConfig::WriteProfileVideoProcAmpXML(XmlParser& root, CVideoProcAmpConfig* vpa)
{
	for (int i = 0; i < 10; ++i)
	{
		auto x = root.put(videoProcAmpKey[i]);
		x.put(L"value", vpa[i].lValue);
		x.put(L"flags", vpa[i].lFlags);
	}
}