#include "StdAfx.h"
#include "CustomTee.h"

#define TEMPLATE_NAME			(L"CustomTee")
#define FILTER_NAME				(TEMPLATE_NAME)
#define VIDEO_INPUT_PIN_NAME	(L"Video In")
#define AUDIO_INPUT_PIN_NAME	(L"Audio In")
#define VIDEO_OUTPUT_PIN_NAME	(L"Viode Out")
#define AUDIO_OUTPUT_PIN_NAME	(L"Audio Out")


const AMOVIESETUP_MEDIATYPE sudVideoPinTypes =
{
	&MEDIATYPE_Video, &MEDIASUBTYPE_NULL
};

const AMOVIESETUP_MEDIATYPE sudAudioPinTypes =
{
	&MEDIATYPE_Audio, &MEDIASUBTYPE_NULL
};

const AMOVIESETUP_PIN sudPins[4]=
{
	{
		VIDEO_INPUT_PIN_NAME,
		FALSE,
		FALSE,//̓s
		FALSE,
		FALSE,
		&CLSID_NULL,
		NULL,
		1,
		&sudVideoPinTypes
	},
	{
		AUDIO_INPUT_PIN_NAME,
		FALSE,
		FALSE,//̓s
		FALSE,
		FALSE,
		&CLSID_NULL,
		NULL,
		1,
		&sudVideoPinTypes
	},
	{
		VIDEO_OUTPUT_PIN_NAME,
		FALSE,
		TRUE,//o̓s
		FALSE,
		FALSE,
		&CLSID_NULL,
		NULL,
		1,
		&sudAudioPinTypes
	},
	{
		AUDIO_OUTPUT_PIN_NAME,
		FALSE,
		TRUE,//o̓s
		FALSE,
		FALSE,
		&CLSID_NULL,
		NULL,
		1,
		&sudAudioPinTypes
	},
};

// tB^
const AMOVIESETUP_FILTER afFilterInfo =
{
	&__uuidof(CCustomTeeFilter),// tB^CLSID
	FILTER_NAME,	 // tB^
	MERIT_DO_NOT_USE,// bgl
	4,               // s
	sudPins         // s
};


// =========================================================================
// CCustomTeeVideoInputPin
// =========================================================================
CCustomTeeVideoInputPin::CCustomTeeVideoInputPin(CBaseFilter *pFilter, CCritSec *pLock, HRESULT *phr)
	: CBaseInputPin(INPUT_PIN_NAME, pFilter, pLock, phr, INPUT_PIN_NAME)
{
}

CCustomTeeVideoInputPin::~CCustomTeeVideoInputPin()
{
}

STDMETHODIMP CCustomTeeVideoInputPin::BeginFlush(void)
{
	CAutoLock lock_it(m_pLock);

	__super::BeginFlush();

	CCustomTeeFilter* pFilter = (CCustomTeeFilter *)m_pFilter;
	pFilter->GetVideoOutputPin()->DeliverBeginFlush();

	return S_OK;
}

STDMETHODIMP CCustomTeeVideoInputPin::EndFlush(void)
{
	CAutoLock my_lock(m_pLock);

	CCustomTeeFilter* pFilter = (CCustomTeeFilter *)m_pFilter;
	pFilter->GetVideoOutputPin()->DeliverEndFlush();

	__super::EndFlush();

	return S_OK;
}

STDMETHODIMP CCustomTeeVideoInputPin::NewSegment
	(REFERENCE_TIME tStart, REFERENCE_TIME tStop, double dRate)
{
	CAutoLock my_lock(m_pLock);

	__super::NewSegment(tStart, tStop, dRate);

	CCustomTeeFilter* pFilter = (CCustomTeeFilter *)m_pFilter;
	pFilter->GetVideoOutputPin()->DeliverNewSegment(tStart, tStop, dRate);

	return S_OK;
}

STDMETHODIMP CCustomTeeVideoInputPin::EndOfStream(void)
{
	HRESULT hr = CheckStreaming();
	if (hr != S_OK)
	{
		return hr;
	}

	CCustomTeeFilter* pFilter = (CCustomTeeFilter *)m_pFilter;
	pFilter->GetVideoOutputPin()->DeliverEndOfStream();

	return S_OK;
}
STDMETHODIMP CCustomTeeVideoInputPin::Receive(IMediaSample *pSample)
{
	HRESULT hr = __super::Receive(pSample);
	if (hr != S_OK)
	{
		return hr;
	}

	CCustomTeeFilter* pFilter = (CCustomTeeFilter *)m_pFilter;
	if (pFilter->GetVideoOutputPin()->IsConnected() == FALSE)
	{
		return hr;
	}

	REFERENCE_TIME rtStartTime, rtEndTime;
	hr = pSample->GetTime(&rtStartTime, &rtEndTime);
	if (SUCCEEDED(hr))
	{
	}
	else
	{
	}


	LPBYTE copy_src, copy_dest;
	REFERENCE_TIME start_time_in, end_time_in, dur;
	LONGLONG start_media_time, end_media_time;
	HRESULT get_time_hr=
		pSample->GetTime(&start_time_in, &end_time_in);
	dur=end_time_in-start_time_in;
	HRESULT get_media_time_hr=
		pSample->GetMediaTime(&start_media_time, &end_media_time);
	pSample->GetPointer(&copy_src);
	long input_sample_length=pSample->GetActualDataLength();
	/*
	static long ttl;
	ttl += input_sample_length;
	NOTE1("ttl=%ld", ttl);
	*/
	long read_count=0, write_count=0;
	WAVEFORMATEXTENSIBLE *wfxe=(WAVEFORMATEXTENSIBLE *)m_mt.pbFormat;
	AudioChSplitterFilter *filter=
		(AudioChSplitterFilter*)m_pFilter;
	while (input_sample_length-read_count>0) {
		long write_size; //ރTCY
		CComPtr<IMediaSample> out_sample[2];
		for(int i=0;i<2;++i) {
			if(filter->GetOutPin(i)->IsConnected()==FALSE)
				continue;
			//NOTE1("GetDeliveryBuffer -> %d", read_count);
			hr=filter->GetOutPin(i)->GetDeliveryBuffer(
				&out_sample[i], NULL, NULL, 0);
			if(FAILED(hr)) {
				break;
			}
			//NOTE1("GetDeliveryBuffer <- %x", hr);
			out_sample[i]->GetPointer(&copy_dest);
			long out_sample_length=out_sample[i]->GetSize();
			write_size=min(
				 (input_sample_length-read_count)/2
				,out_sample_length);
			long sample_count=
				write_size/(wfxe->Format.nBlockAlign/2);
			int offset=i*wfxe->Format.nBlockAlign/2;
			for(int n=0;n<sample_count;++n) {
				CopyMemory(copy_dest
					, copy_src+offset+wfxe->Format.nBlockAlign*n
					, wfxe->Format.nBlockAlign/2);
				copy_dest+=(wfxe->Format.nBlockAlign/2);
			}
			// Tv̎f[^
			hr=out_sample[i]->SetActualDataLength(write_size);
			// fBA^C
			if(SUCCEEDED(get_media_time_hr)) {
				end_media_time=start_media_time+sample_count;
				out_sample[i]->SetMediaTime(
					&start_media_time, &end_media_time);
				start_media_time=end_media_time;
			}else {
				out_sample[i]->SetMediaTime(NULL, NULL);
			}
			// Xg[^C
			if(SUCCEEDED(get_time_hr)) {
				int sample_sum=
					write_count/wfxe->Format.nBlockAlign;
				REFERENCE_TIME start_time=
					start_time_in+
					sample_sum*UNITS/wfxe->Format.nSamplesPerSec;
				REFERENCE_TIME end_time  =
					start_time+write_size/(wfxe->Format.nBlockAlign/2)
					*UNITS/wfxe->Format.nSamplesPerSec;
				out_sample[i]->SetTime(&start_time, &end_time);
			}else {
				out_sample[i]->SetTime(NULL, NULL);
			}
			// ̑
			out_sample[i]->SetPreroll(FALSE);
			out_sample[i]->SetDiscontinuity(FALSE);
			out_sample[i]->SetSyncPoint(TRUE);
			//_EXg[tB^֑M
			hr=filter->GetOutPin(i)->Deliver(out_sample[i]);
			if(FAILED(hr)) {
				continue;
			}
		}
		read_count +=write_size*2;
		write_count+=write_size;
	}
	return hr;
}

HRESULT CCustomTeeVideoInputPin::GetMediaType(int iPosition, CMediaType *pMediaType)
{
	if (iPosition < 0)
	{
		return E_INVALIDARG;
	}
	return VFW_S_NO_MORE_ITEMS;
}

HRESULT CCustomTeeVideoInputPin::CheckMediaType(const CMediaType *pmt)
{
	if (pmt->majortype == MEDIATYPE_Video)
	{
		if(pmt->Format()==NULL)
			return S_FALSE;
		if(pmt->FormatLength()!=sizeof(WAVEFORMATEXTENSIBLE) &&
			pmt->FormatLength()!=sizeof(WAVEFORMATEX))
		{
			return S_FALSE;
		}
		WAVEFORMATEXTENSIBLE *wfxe=
			(WAVEFORMATEXTENSIBLE *)pmt->Format();
		if((wfxe->Format.wFormatTag==WAVE_FORMAT_EXTENSIBLE ||
			wfxe->Format.wFormatTag==WAVE_FORMAT_PCM) &&
			wfxe->Format.nChannels==2)
		{
			return S_OK;
		}
		return S_FALSE;
	}
	return S_FALSE;
}
