#include "StdAfx.h"
#include "Misc.h"

BOOL GetDesktopRectFromPoint(const POINT pt, RECT &rc)
{
	HMONITOR mon = ::MonitorFromPoint(pt, MONITOR_DEFAULTTOPRIMARY);

	if (mon != NULL)
	{
		MONITORINFO mi;
		mi.cbSize = sizeof(MONITORINFO);
		if (::GetMonitorInfo(mon, &mi))
		{
			rc = mi.rcMonitor;
			return TRUE;
		}
	}

	return FALSE;
}

BOOL GetDesktopRectFromRect(const RECT &rcFrom, RECT &rc)
{
	HMONITOR mon = ::MonitorFromRect(&rcFrom, MONITOR_DEFAULTTOPRIMARY);

	if (mon != NULL)
	{
		MONITORINFO mi;
		mi.cbSize = sizeof(MONITORINFO);
		if (::GetMonitorInfo(mon, &mi))
		{
			rc = mi.rcMonitor;
			return TRUE;
		}
	}

	return FALSE;
}

BOOL GetDesktopRectFromWindow(const HWND hWnd, RECT &rc)
{
	HMONITOR mon = ::MonitorFromWindow(hWnd, MONITOR_DEFAULTTOPRIMARY);

	if (mon != NULL)
	{
		MONITORINFO mi;
		mi.cbSize = sizeof(MONITORINFO);
		if (::GetMonitorInfo(mon, &mi))
		{
			rc = mi.rcMonitor;
			return TRUE;
		}
	}

	return FALSE;
}


////////////////////////////////////////
// fXNgbv֐
BOOL getDesktopRectFromPoint(const POINT pt, MONITORINFO &mi)
{
	HMONITOR mon = ::MonitorFromPoint(pt, MONITOR_DEFAULTTOPRIMARY);

	if (mon != NULL)
	{
		mi.cbSize = sizeof(MONITORINFO);
		if (::GetMonitorInfo(mon, &mi))
		{
			return TRUE;
		}
	}

	return FALSE;
}

BOOL getDesktopRectFromRect(const RECT &rcFrom, MONITORINFO &mi)
{
	HMONITOR mon = ::MonitorFromRect(&rcFrom, MONITOR_DEFAULTTOPRIMARY);

	if (mon != NULL)
	{
		mi.cbSize = sizeof(MONITORINFO);
		if (::GetMonitorInfo(mon, &mi))
		{
			return TRUE;
		}
	}

	return FALSE;
}

BOOL getDesktopRectFromWindow(const HWND hWnd, MONITORINFO &mi)
{
	HMONITOR mon = ::MonitorFromWindow(hWnd, MONITOR_DEFAULTTOPRIMARY);

	if (mon != NULL)
	{
		mi.cbSize = sizeof(MONITORINFO);
		if (::GetMonitorInfo(mon, &mi))
		{
			return TRUE;
		}
	}

	return FALSE;
}


void setWindowSnap(HWND hwnd, LPPOINT lppt, int distance)
{
	if (hwnd == NULL) { return; }
	
	bool move = false;
	MONITORINFO mi;
	CRect desktop, client, org, shape;

	GetWindowRect(hwnd, &org);
	GetWindowShapeRect(hwnd, &shape);

	LONG left = shape.left - org.left;
	LONG top = shape.top - org.top;

	lppt->x += left;
	lppt->y += top;

	client.left = lppt->x - left;
	client.top = lppt->y - top;
	client.right = lppt->x + shape.Width();
	client.bottom = lppt->y + shape.Height();

	if (getDesktopRectFromWindow(hwnd, mi))
	{
		// [NGA(^XNo[OTCY)
		desktop = mi.rcWork;

		desktop.left -= left;
		desktop.top -= top;
		
		if (client.left < (desktop.left + distance) &&
			client.left > (desktop.left - distance) &&
			client.top < (desktop.top + distance) &&
			client.top > (desktop.top - distance))
		{
			SetWindowPos(hwnd, NULL, desktop.left, desktop.top, 0, 0, SWP_NOSIZE|SWP_NOZORDER);
			return;
		}

		if (client.left < (desktop.left + distance) &&
			client.left > (desktop.left - distance) &&
			client.bottom < (desktop.bottom + distance) &&
			client.bottom > (desktop.bottom - distance))
		{
			SetWindowPos(hwnd, NULL, desktop.left, desktop.bottom - client.Height(), 0, 0, SWP_NOSIZE|SWP_NOZORDER);
			return;
		}

		if (client.right < (desktop.right + distance) &&
			client.right > (desktop.right - distance) &&
			client.top < (desktop.top + distance) &&
			client.top > (desktop.top - distance))
		{
			SetWindowPos(hwnd, NULL, desktop.right - client.Width(), desktop.top, 0, 0, SWP_NOSIZE|SWP_NOZORDER);
			return;
		}

		if (client.right < (desktop.right + distance) &&
			client.right > (desktop.right - distance) &&
			client.bottom < (desktop.bottom + distance) &&
			client.bottom > (desktop.bottom - distance))
		{
			SetWindowPos(hwnd, NULL, desktop.right - client.Width(), desktop.bottom - client.Height(), 0, 0, SWP_NOSIZE|SWP_NOZORDER);
			return;
		}


		if (client.left < (desktop.left + distance) &&
			client.left > (desktop.left - distance))
		{
			SetWindowPos(hwnd, NULL, desktop.left, client.top, 0, 0, SWP_NOSIZE|SWP_NOZORDER);
			return;
		}
		
		if (client.right > (desktop.right - distance) &&
			client.right < (desktop.right + distance))
		{
			SetWindowPos(hwnd, NULL, desktop.right - client.Width(), client.top, 0, 0, SWP_NOSIZE|SWP_NOZORDER);
			return;
		}

		if (client.top < (desktop.top + distance) &&
			client.top > (desktop.top - distance))
		{
			SetWindowPos(hwnd, NULL, client.left, desktop.top, 0, 0, SWP_NOSIZE|SWP_NOZORDER);
			return;
		}
		
		if (client.bottom > (desktop.bottom - distance) &&
			client.bottom < (desktop.bottom + distance))
		{
			SetWindowPos(hwnd, NULL, client.left, desktop.bottom - client.Height(), 0, 0, SWP_NOSIZE|SWP_NOZORDER);
			return;
		}
	}
	SetWindowPos(hwnd, NULL, client.left, client.top, 0, 0, SWP_NOSIZE|SWP_NOZORDER);
}





HMODULE ModuleLoader::module_ = NULL;
ModuleLoader::PDwmIsCompositionEnabled ModuleLoader::fpDwmIsCompositionEnabled = NULL;
ModuleLoader::PDwmGetWindowAttribute ModuleLoader::fpDwmGetWindowAttribute = NULL;



ModuleLoader::ModuleLoader()
{
	if (module_ == NULL)
	{
		TCHAR dir[MAX_PATH];
		GetSystemDirectory(dir, sizeof(dir));

		lstrcat(dir, _T("\\dwmapi.dll"));

		module_ = ::LoadLibrary(dir);
		if (module_)
		{
			fpDwmIsCompositionEnabled = (PDwmIsCompositionEnabled)GetProcAddress(module_, "DwmIsCompositionEnabled");
			fpDwmGetWindowAttribute = (PDwmGetWindowAttribute)GetProcAddress(module_, "DwmGetWindowAttribute");
		}
	}
}

ModuleLoader::~ModuleLoader()
{
	if (module_) { ::FreeLibrary(module_); }
}

void ModuleLoader::Init()
{
	static ModuleLoader instance;
}

HRESULT WINAPI ModuleLoader::DwmIsCompositionEnabled(_Out_ BOOL *pfEnabled)
{
	if (!fpDwmIsCompositionEnabled) { return E_NOTIMPL; }
	return fpDwmIsCompositionEnabled(pfEnabled);
}

HRESULT WINAPI ModuleLoader::DwmGetWindowAttribute(HWND hwnd, DWORD dwAttribute, _Out_ PVOID pvAttribute, DWORD cbAttribute)
{
	if (!fpDwmGetWindowAttribute) { return E_NOTIMPL; }
	return fpDwmGetWindowAttribute(hwnd, dwAttribute, pvAttribute, cbAttribute);
}



BOOL GetWindowShapeRect(HWND hwnd, LPRECT lprc)
{
	BOOL enable = FALSE;
	HRESULT hr = ModuleLoader::DwmIsCompositionEnabled(&enable);
	if (SUCCEEDED(hr))
	{
		if (enable)
		{
			hr = ModuleLoader::DwmGetWindowAttribute(hwnd, DWMWA_EXTENDED_FRAME_BOUNDS, lprc, sizeof(RECT));
			if (SUCCEEDED(hr))
			{
				return TRUE;
			}
		}
	}
	return GetWindowRect(hwnd, lprc);
}