#include "StdAfx.h"
#include "SnapShot.h"

CSnapShot::CSnapShot(void)
{
}

CSnapShot::~CSnapShot(void)
{
}

void CSnapShot::CreateFileName(long lFormat, std::wstring &wsFileName)
{
	SYSTEMTIME time;
	::memset(&time, 0, sizeof(SYSTEMTIME));
	::GetLocalTime(&time);

	CString csTmp;
	csTmp.Format(L"%04d_%02d%02d_%02d%02d%02d_%03d",
		time.wYear, time.wMonth, time.wDay, time.wHour, time.wMinute, time.wSecond, time.wMilliseconds);

	wsFileName = csTmp;

	switch (lFormat)
	{
	case FORMAT_JPG:
		wsFileName += L".jpg";
		break;
	case FORMAT_PNG:
		wsFileName += L".png";
		break;
	case FORMAT_BMP:
	default:
		wsFileName += L".bmp";
		break;
	}
}

BOOL CSnapShot::Save(
	LPCWSTR pwszFolderPath, long lFormat,
	BITMAPINFO *pBmpInfo, BYTE *pDIB,
	int nWidth, int nHeight,
	LPRECT prcCrop,
	ULONG ulJpegQuality)
{
	if (pwszFolderPath == NULL)
	{
		return FALSE;
	}

	Bitmap *pSrc = Bitmap::FromBITMAPINFO(pBmpInfo, pDIB);

	if (pSrc == NULL)
	{
		return FALSE;
	}

	if (nWidth == 0 || nHeight == 0)
	{
		nWidth = pSrc->GetWidth();
		nHeight = pSrc->GetHeight();
	}

	Bitmap bmp(nWidth, nHeight, pSrc->GetPixelFormat());

	Graphics g(&bmp);

	Rect rcDst;
	rcDst.X = 0;
	rcDst.Y = 0;
	rcDst.Width = nWidth;
	rcDst.Height = nHeight;

	g.SetSmoothingMode(SmoothingModeNone);
	g.SetInterpolationMode(InterpolationModeHighQualityBicubic);

	if (prcCrop == NULL)
	{
		g.DrawImage(pSrc, rcDst, 0, 0, pSrc->GetWidth(), pSrc->GetHeight(), UnitPixel);
	}
	else
	{
		g.DrawImage(
			pSrc, rcDst,
			prcCrop->left,
			prcCrop->top,
			pSrc->GetWidth() - (prcCrop->left + prcCrop->right),
			pSrc->GetHeight() - (prcCrop->top + prcCrop->bottom),
			UnitPixel);
	}


	EncoderParameters *eps = NULL;
	ULONG quality = ulJpegQuality; // 0`100
	ULONG compress = 100; // 0`100
	ULONG colordepth = 32; // 1, 4, 8, 24, 32

	if (lFormat == CSnapShot::FORMAT_JPG)
	{
		eps = static_cast<EncoderParameters*>(
			malloc(sizeof(EncoderParameters))
			);

		if (eps == NULL)
		{
			delete pSrc;
			return FALSE;
		}

		eps->Count = 1;
		eps->Parameter[0].Guid = EncoderQuality;
		eps->Parameter[0].Type = EncoderParameterValueTypeLong;
		eps->Parameter[0].NumberOfValues = 1;
		eps->Parameter[0].Value = &quality;
	}
	else if (lFormat == CSnapShot::FORMAT_PNG)
	{
		eps = static_cast<EncoderParameters*>(
			malloc(sizeof(EncoderParameters) + sizeof(EncoderParameter))
			);

		if (eps == NULL)
		{
			delete pSrc;
			return FALSE;
		}

		eps->Count = 2;

		//EncoderParameters* encoderParameters = (EncoderParameters*)malloc(sizeof( EncoderParameters) + 4 * sizeof(EncoderParameter));

		eps->Parameter[0].Guid = EncoderCompression;
		eps->Parameter[0].Type = EncoderParameterValueTypeLong;
		eps->Parameter[0].NumberOfValues = 1;
		eps->Parameter[0].Value = &compress;

		eps->Parameter[1].Guid = EncoderColorDepth;
		eps->Parameter[1].Type = EncoderParameterValueTypeLong;
		eps->Parameter[1].NumberOfValues = 1;
		eps->Parameter[1].Value = &colordepth;
	}
	else
	{
		eps = static_cast<EncoderParameters*>(
			malloc(sizeof(EncoderParameters))
			);

		if (eps == NULL)
		{
			delete pSrc;
			return FALSE;
		}

		eps->Count = 1;

		eps->Parameter[0].Guid = EncoderColorDepth;
		eps->Parameter[0].Type = EncoderParameterValueTypeLong;
		eps->Parameter[0].NumberOfValues = 1;
		eps->Parameter[0].Value = &colordepth;
	}

	CLSID clsid;
    if (GetEncoderClsid(lFormat, &clsid))
    {
		DWORD dwError;
		dwError = ::SHCreateDirectory(NULL, pwszFolderPath);
		if (ERROR_ALREADY_EXISTS != dwError &&
			ERROR_FILE_EXISTS != dwError &&
			ERROR_SUCCESS != dwError)
		{
			free(eps);
			delete pSrc;
			return FALSE;
		}

		std::wstring wsFileName, wsFilePath;
		CreateFileName(lFormat, wsFileName);

		wsFilePath = pwszFolderPath;
		wsFilePath += wsFileName;

		bmp.Save(wsFilePath.c_str(), &clsid, eps);
		free(eps);
		delete pSrc;
        return TRUE;
    }

	free(eps);
	delete pSrc;
    return FALSE;
}

BOOL CSnapShot::GetEncoderClsid(const WCHAR* format, CLSID* pClsid)
{
	Status stat;
    UINT num = 0, size = 0;


    stat = GetImageEncodersSize(&num, &size);
    if (stat != Ok || size == 0)
	{
        return FALSE;
	}

    ImageCodecInfo* pImageCodecInfo = (ImageCodecInfo*)malloc(size);
    if (pImageCodecInfo == NULL)
	{
        return FALSE;
	}

    stat = GetImageEncoders(num, size, pImageCodecInfo);
	if (stat != Ok)
	{
		free(pImageCodecInfo);
		return FALSE;
	}

    for(UINT i = 0; i < num; ++i)
    {
        if (wcscmp(pImageCodecInfo[i].MimeType, format) == 0)
        {
            *pClsid = pImageCodecInfo[i].Clsid;
            free(pImageCodecInfo);
            return TRUE;
        }
    }

    free(pImageCodecInfo);
    return FALSE;
}

BOOL CSnapShot::GetEncoderClsid(long lFormat, CLSID* pClsid)
{
	BOOL bResult;

	switch (lFormat)
	{
	case FORMAT_JPG:
		bResult = GetEncoderClsid(L"image/jpeg", pClsid);
		break;
	case FORMAT_PNG:
		bResult = GetEncoderClsid(L"image/png", pClsid);
		break;
	case FORMAT_BMP:
	default:
		bResult = GetEncoderClsid(L"image/bmp", pClsid);
		break;
	}

	return bResult;
}

