#pragma once

#pragma warning(disable : 4005)
#pragma warning(disable : 4503)

#include <array>
#include <vector>
#include <string>
#include <functional>
#include <codecvt>

#include <boost/tokenizer.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/optional.hpp>
#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/xml_parser.hpp>

#define DText(str, ...) DebugText(str, __VA_ARGS__)
//#define DText(x)


class XmlParser {
	typedef boost::optional<boost::property_tree::wptree&> optree;

	boost::property_tree::wptree pt_;
	optree opt_;

public:
	XmlParser()
		: opt_(pt_)
	{}

	XmlParser(const optree& opt)
		: opt_(opt)
	{}

	XmlParser(const XmlParser& x)
		: opt_(x.opt_)
	{}

	XmlParser(const std::wstring& path)
		: opt_(pt_)
	{
		read(path);
	}

	XmlParser operator=(const XmlParser& x)
	{
		opt_ = x.opt_;
		return *this;
	}

	operator bool() // VKǉ
	{
		return opt_;
	}

	////////////////////////////////////////
	// Writer
	void write(const std::wstring& path)
	{
		using namespace boost::property_tree;

		FILE *file = nullptr;
		if (!::_wfopen_s(&file, path.c_str(), L"wb"))
		{
			std::wstringstream wss;
			try {
				write_xml(wss, pt_, xml_writer_make_settings<std::wstring>(L'\t', 1));
			}
			catch (...) {}

			std::string cnv = toUTF8(wss.str());

			fwrite(cnv.c_str(), sizeof(char), cnv.size(), file);
			
			::fclose(file);
		}
	}

	// ėp
	template<typename T>
	void put(const std::wstring& path, const T& value)
	{
		if (opt_)
		{
			opt_->put(L"<xmlattr>." + path, value);
		}
	}

	// vector
	template<typename T>
	void put(const std::wstring& path, const std::vector<T>& value)
	{
		put(path, makeCsv(value));
	}

	// array
	template<typename T, size_t SIZE>
	void put(const std::wstring& path, const std::array<T, SIZE>& value)
	{
		put(path, makeCsv(value));
	}

	// ew
	XmlParser put(const std::wstring& path)
	{
		if (opt_)
		{
			return opt_->put(path, L"");
		}
		return boost::none;
	}

	// eǉ
	XmlParser add(const std::wstring& path)
	{
		if (opt_)
		{
			return opt_->add(path, L"");
		}
		return boost::none;
	}

	////////////////////////////////////////
	// Reader
	void read(const std::wstring& path)
	{
		using namespace boost::property_tree;
		std::wifstream ifs(path);
		if (ifs.is_open())
		{
			ifs.imbue(std::locale(std::locale::empty(), new std::codecvt_utf8<wchar_t>));
			try {
				read_xml(ifs, pt_);
			}
			catch (...) {}
			ifs.close();
		}
	}

	// ėpO
	template<typename T>
	bool get(const std::wstring& path, T& value) const
	{
		if (opt_)
		{
			auto v = opt_->get_optional<T>(L"<xmlattr>." + path);
			if (v)
			{
				value = *v;
				return true;
			}
		}
		DText(L"[%s] template<typename T> bool get(const std::wstring& path, T& value) const\n", path.c_str());
		throw std::exception();
		return false;
	}

	// vectorO
	template<typename T>
	bool get(const std::wstring& path, std::vector<T>& value) const
	{
		if (opt_)
		{
			auto v = opt_->get_optional<std::wstring>(L"<xmlattr>." + path);
			if (v)
			{
				readCsv(value, *v);
				return true;
			}
		}
		DText(L"[%s] template<typename T> bool get(const std::wstring& path, std::vector<T>& value) const\n", path.c_str());
		throw std::exception();
		return false;
	}

	// arrayO
	template<typename T, size_t SIZE>
	bool get(const std::wstring& path, std::array<T, SIZE>& value) const
	{
		if (opt_)
		{
			auto v = opt_->get_optional<std::wstring>(L"<xmlattr>." + path);
			if (v)
			{
				readCsv(value, *v);
				return true;
			}
		}
		DText(L"[%s] template<typename T, size_t SIZE> bool get(const std::wstring& path, std::array<T, SIZE>& value) const\n", path.c_str());
		throw std::exception();
		return false;
	}

	// ėp
	template<typename T>
	bool get(const std::wstring& path, T& value, const T& def) const
	{
		if (opt_)
		{
			auto v = opt_->get_optional<T>(L"<xmlattr>." + path);
			if (v)
			{
				value = *v;
				return true;
			}
		}
		DText(L"[%s] template<typename T> bool get(const std::wstring& path, T& value, const T& def) const\n", path.c_str());
		value = def;
		return false;
	}

	// vector
	template<typename T>
	bool get(const std::wstring& path, std::vector<T>& value, const wchar_t* def) const
	{
		if (opt_)
		{
			auto v = opt_->get_optional<std::wstring>(L"<xmlattr>." + path);
			if (v)
			{
				readCsv(value, *v);
				return true;
			}
		}
		DText(L"[%s] template<typename T> bool get(const std::wstring& path, std::vector<T>& value, const wchar_t* def) const\n", path.c_str());
		readCsv(value, def);
		return false;
	}

	// array
	template<typename T, size_t SIZE>
	bool get(const std::wstring& path, std::array<T, SIZE>& value, const wchar_t* def) const
	{
		if (opt_)
		{
			auto v = opt_->get_optional<std::wstring>(L"<xmlattr>." + path);
			if (v)
			{
				readCsv(value, *v);
				return true;
			}
		}
		DText(L"[%s] template<typename T> bool get(const std::wstring& path, std::array<T, SIZE>& value, const wchar_t* def) const\n", path.c_str());
		readCsv(value, def);
		return false;
	}

	// p
	bool get(const std::wstring& path, std::wstring& value, const wchar_t* def) const
	{
		return get(path, value, std::wstring(def));
	}

	// bool->BOOL I[o[[h//VKǉ
	bool get(const std::wstring& path, BOOL& value, bool def) const
	{
		if (opt_)
		{
			auto v = opt_->get_optional<bool>(L"<xmlattr>." + path);
			if (v)
			{
				value = (*v) ? TRUE : FALSE;
				return true;
			}
		}
		DText(L"[%s] bool get(const std::wstring& path, BOOL& value, bool def) const\n", path.c_str());
		value = def ? TRUE : FALSE;
		return false;
	}

	// ew
	XmlParser get(const std::wstring& path) const
	{
		if (opt_)
		{
			return opt_->get_child_optional(path);
		}
		DText(L"[%s] XmlParser get(const std::wstring& path) const\n", path.c_str());
		return boost::none;
	}


	// 
	void foreach(const std::wstring& path, const std::function<void(const XmlParser&)>& func) const
	{
		if (opt_)
		{
			for (auto it = opt_->begin(); it != opt_->end(); ++it)
			{
				if (it->first == path)
				{
					func(XmlParser(it->second));
				}
			}
		}
	}


private:

	// ReiCSVɕϊ
	template<typename T>
	std::wstring makeCsv(const T& v)
	{
		std::wstringstream ss;
		for (auto it = v.begin(); it != v.end(); ++it)
		{
			if (it != v.begin()) { ss << L","; }
			ss << *it;
		}
		return ss.str();
	}

	// vector
	template<typename T>
	std::wstring makeCsv(const std::vector<T>& v)
	{
		return makeCsv<std::vector<T> >(v);
	}

	// array
	template<typename T, size_t SIZE>
	std::wstring makeCsv(const std::array<T, SIZE>& v)
	{
		return makeCsv<std::array<T, SIZE> >(v);
	}

	// csvvectorɓǂݍ
	template<typename T>
	void readCsv(std::vector<T>& v, const std::wstring &str)
	{
		typedef boost::char_separator<wchar_t> separator;
		typedef boost::tokenizer<separator, std::wstring::const_iterator, std::wstring> tokenizer;

		separator sep(L",", L"", boost::keep_empty_tokens);
		tokenizer tokens(str, sep);

		for (auto it = tokens.begin(); it != tokens.end(); ++it)
		{
			v.push_back(boost::lexical_cast<T>(it->c_str()));
		}
	}

	// csvarrayɓǂݍ
	template<typename T, size_t SIZE>
	void readCsv(std::array<T, SIZE>& v, const std::wstring &str)
	{
		typedef boost::char_separator<wchar_t> separator;
		typedef boost::tokenizer<separator, std::wstring::const_iterator, std::wstring> tokenizer;

		separator sep(L",", L"", boost::keep_empty_tokens);
		tokenizer tokens(str, sep);

		int i = 0;
		for (auto it = tokens.begin(); it != tokens.end(); ++it)
		{
			if (i == v.size()) { break; }
			v[i] = boost::lexical_cast<T>(it->c_str());
			++i;
		}
	}



	// UTF16UTF8֕ϊ
	std::string toUTF8(const std::wstring& src)
	{
		const int size = ::WideCharToMultiByte(CP_UTF8, 0, src.c_str(), -1, NULL, 0, NULL, NULL);

		if (size < 0) { return std::string(); }

		std::vector<char> buff(size);
	
		int result = ::WideCharToMultiByte(CP_UTF8, 0, src.c_str(), -1, buff.data(), size, NULL, NULL);

		if (result < 0) { return std::string(); }

		return buff.data();
	}
};



// xml̓ǂݏ
/*


{// 
	XmlParser xml;
	auto root = xml.put(L"root");
	auto x = root.put(L"window.size");
	x.put(L"x", 640);
	x.put(L"y", 480);
	xml.write(L"test.xml");
}

{// ǂݍ
	int x, y;
	XmlParser xml(L"test.xml");
	auto root = xml.get(L"root");

	// w肵pXȂꍇ x ͖ɂȂ
	// ȍ~ x.get ɎŝŎw肵lŏ
	auto x = root.get(L"window.size");
	x.get(L"x", x, 640);
	x.get(L"y", y, 480);


	// w肵pXȂꍇ x ͖ɂȂ
	// x == false ɂȂ
	x = root.get(L"size");
	if (x)
	{// size ȉ̕ item 𑖍
		x.foreach(L"item", [&](const XmlParser& item){
			BOOL enable;
			int width, height;

			item.get(L"enable", enable, false);
			item.get(L"width", width, 800);
			item.get(L"height", height, 600);

			vecWindowSize_.push_back(WindowSize(enable, width, height));
		});
	}
	else
	{
		vecWindowSize_.push_back(WindowSize(TRUE, 1920, 1080));
		vecWindowSize_.push_back(WindowSize(FALSE, 1600, 900));
		vecWindowSize_.push_back(WindowSize(FALSE, 1440, 810));
	}
}


// O̎g
	try {
		p = root.get(L"windowPlacement");
		p.get(L"length", windowPlacement.length);
		p.get(L"flags", windowPlacement.flags);
		p.get(L"showCmd", windowPlacement.showCmd);

		p = root.get(L"windowPlacement.ptMinPosition");
		p.get(L"x", windowPlacement.ptMinPosition.x);
		p.get(L"y", windowPlacement.ptMinPosition.y);

		p = root.get(L"windowPlacement.ptMaxPosition");
		p.get(L"x", windowPlacement.ptMaxPosition.x);
		p.get(L"y", windowPlacement.ptMaxPosition.y);

		p = root.get(L"windowPlacement.rcNormalPosition");
		p.get(L"left", windowPlacement.rcNormalPosition.left);
		p.get(L"top", windowPlacement.rcNormalPosition.top);
		p.get(L"right", windowPlacement.rcNormalPosition.right);
		p.get(L"bottom", windowPlacement.rcNormalPosition.bottom);
	}
	catch (...) {
		windowPlacement = utl::defWindowPlacement(640, 480);
	}

*/

